/* -----------------------------------------------------------------------
 *
 * Project:     UT-SNMP
 *
 * Filename:    apiTYPES.h
 *
 * Date:        June 1994
 *
 * Purpose:     Include for UT_SNMP application.
 *
 * Author(s):   Vincent Berkhout, Frederic P. H. van Hengstum
 *
 * -----------------------------------------------------------------------
 */

#ifndef APITYPES_H
#define APITYPES_H


extern AsnOcts *mOcNew( unsigned int l );
extern AsnOcts *mOcPut( AsnOcts *p, unsigned char *s, unsigned int l );
extern AsnOcts *mOcGet( AsnOcts *p, unsigned char *s, unsigned int *l );
extern AsnBits *mBiNew( int l );
extern AsnBits *mBiPut( AsnBits *p, char *s, int l );
extern AsnBits *mBiGet( AsnBits *p, char *s, int *l );
extern ObjectSyntax *mObNew( );
extern ObjectSyntax *mObPut( ObjectSyntax *p, ObjectSyntaxChoiceId i, AsnInt v );
extern AsnInt mObGet( ObjectSyntax *p, ObjectSyntaxChoiceId *i);
extern VarBindChoice *mVcPut( VarBindChoice *p, VarBindChoiceChoiceId i, 
							  ObjectSyntax *v );
extern ObjectSyntax *mVcGet( VarBindChoice *p, VarBindChoiceChoiceId *i );
extern VarBind *mVbNew( unsigned int l );
extern VarBind *mVbPut( VarBind *p, ObjectName *n, VarBindChoice *v );
extern VarBindChoice *mVbGet( VarBind *p, ObjectName *n );

/* -----------------------------------------------------------------------
 *
 *	typedef struct {
 *		unsigned long int octetLen;
 *		unsigned char* octs;
 *	} AsnOcts;
 *
 * -----------------------------------------------------------------------
 */
#define mOcFree( p )	(void) FreeAsnOcts((AsnOcts *)p)
#define mOcGet( p,n,o )	mOcGet( (AsnOcts *) p, n, o )
#define mOcPut( p,n,o )	mOcPut( (AsnOcts *) p, n, o )
#define mOcPrint( p )	(void) PrintAsnOcts(stdout,(AsnOcts *)p,0); fflush(stdout)

/* -----------------------------------------------------------------------
 *
 *	typedef AsnOcts AsnOid;
 *
 * -----------------------------------------------------------------------
 */
#define mOiNew( l ) 		(AsnOid *) mOcNew( l )
#define	mOiPut( p, s, l )	(AsnOid *) mOcPut( p, s, l )
#define mOiGet( p, s, l )	(AsnOid *) mOcGet( p, s, l )
#define mOiFree( p )	(void) FreeAsnOid((AsnOid *) p )
#define mOiPrint( p )	(void) PrintAsnOid(stdout,(AsnOid *) p ,0); fflush(stdout)

/* -----------------------------------------------------------------------
 *
 *	typedef AsnOcts IpAddress
 *
 * -----------------------------------------------------------------------
 */
#define	mIpNew( l )		(IpAddress *) mOcNew( l )
#define	mIpPut( p, s, l )	(IpAddress *) mOcPut( p, s, l )
#define	mIpGet( p, s, l )	(IpAddress *) mOcGet( p, s, l )
#define	mIpFree( p )	(void) FreeIpAddress( (IpAddress *) p )
#define mIpPrint( p )	(void) PrintIpAddress(stdout,(IpAddress *) p ,0); fflush(stdout)

/* -----------------------------------------------------------------------
 *
 *	typedef AsnOcts Opaque
 *
 * -----------------------------------------------------------------------
 */
#define	mOpNew( l )		(Opaque *) mOcNew( l )
#define	mOpPut( p, s, l )	(Opaque *) mOcPut( p, s, l )
#define	mOpGet( p, s, l )	(Opaque *) mOcGet( p, s, l )
#define	mOpFree( p )	(void) FreeOpaque( (Opaque *) p )
#define	mOpPrint( p )	(void) PrintOpaque(stdout,(Opaque *) p ,0); fflush(stdout)

/* -----------------------------------------------------------------------
 *
 *	typedef AsnOcts NsapAddress
 *
 * -----------------------------------------------------------------------
 */
#define	mNsNew( l )		(NsapAddress *) mOcNew( l )
#define	mNsPut( p, s, l )	(NsapAddress *) mOcPut( p, s, l )
#define	mNsGet( p, s, l )	(NsapAddress *) mOcGet( p, s, l )
#define mNsFree( p )	(void) FreeNsapAddress( (NsapAddress *) p )
#define mNsPrint( p )	(void) PrintNsapAddress(stdout,(NsapAddress *) p,0); fflush(stdout)

/* -----------------------------------------------------------------------
 *
 *	typedef struct AsnBits {
 *		int bitLen;
 *		char* bits;
 *	} AsnBits;
 *
 * -----------------------------------------------------------------------
 */
#define mBiNew( l ) 	(AsnBits *) mBiNew( l )
#define	mBiPut( p,n,l )	(AsnBits *) mBiPut( p, n, l )
#define	mBiGet( p,n,l )	(AsnBits *) mBiPut( p, n, l )
#define	mBiFree( p )	(void) FreeAsnBits((AsnBits *) p )
#define mBiPrint( p )	(void) PrintAsnBits(stdout,(AsnBits *) p,0); fflush(stdout)

/* -----------------------------------------------------------------------
 *
 *	typedef struct ObjectSyntax {
 *		ObjectSyntaxChoiceId choiceId;
 *		union ObjectSyntaxChoiceUnion {
 *			AsnInt integer_value;
 *			AsnOcts* string_value; 
 *			AsnOid* objectID_value;
 *			AsnBits* bit_value;
 *			IpAddress* ipAddress_value;
 *			Counter32 counter_value;
 *			Gauge32 gauge_value;
 *			TimeTicks timeticks_value;
 *			Opaque* arbitrary_value;
 *			NsapAddress* nsapAddress_value;
 *			Counter64 big_counter_value;
 *			UInteger32 unsigned_integer_value;
 *		} a;
 *	} ObjectSyntax;
 *
 * -----------------------------------------------------------------------
 */
#define	mObNew( )		(ObjectSyntax *) Asn1Alloc( sizeof( ObjectSyntax ) )
#define	mObPut( p,i,v )	(ObjectSyntax *) mObPut( (ObjectSyntax *) p,  \
							(ObjectSyntaxChoiceId) i, (AsnInt) v )
#define mObGet( p, i ) 	(AsnInt) mObGet( p, i )
#define mObFree( p )	(void) FreeObjectSyntax( (ObjectSyntax *) p )
#define mObPrint( p )	(void) PrintObjectSyntax(stdout, (ObjectSyntax *) p,0); fflush(stdout)

/* -----------------------------------------------------------------------
 *
 *	typedef struct VarBindChoice {
 *		VarBindChoiceChoiceId choiceId;
 *		union VarBindChoiceChoiceUnion {
 *			struct ObjectSyntax* value; 
 *			AsnNull unSpecified; 
 *			AsnNull noSuchObject; 
 *			AsnNull noSuchInstance;
 *			AsnNull endOfMibView;
 *		} a;
 *	} VarBindChoice;
 *
 * -----------------------------------------------------------------------
 */
#define	mVcNew( ) 		(VarBindChoice *) Asn1Alloc( sizeof( VarBindChoice ) )
#define	mVcPut( p,i,v ) (VarBindChoice *) mVcPut( p, i, v )
#define	mVcGet( p,i ) 	(ObjectSyntax *) mVcGet( p, i )
#define mVcFree( p )	(void) FreeVarBindChoice( (VarBindChoice *) p )
#define mVcPrint( p )	(void) PrintVarBindChoice(stdout, (VarBindChoice *) p, 0);\
						fflush(stdout)

/* -----------------------------------------------------------------------
 *
 *	typedef struct VarBind {
 *		ObjectName name;
 *		struct VarBindChoice* varBindChoice;
 *	} VarBind;
 *
 * -----------------------------------------------------------------------
 */
#define	mVbNew( l ) 	(VarBind *) mVbNew( l )
#define	mVbPut( p,i,v ) (VarBind *) mVbPut( p, i, (v) )
#define	mVbGet( p,i ) 	(VarBindChoice *) mVbGet( p, (i) )
#define mVbFree( p )	(void) FreeVarBind( (VarBind *) p )
#define	mVbPrint( p )	(void) PrintVarBind( stdout, (VarBind *) p, 0 ); \
						fflush(stdout)

/* -----------------------------------------------------------------------
 *
 *	typedef AsnList VarBindList
 *
 * -----------------------------------------------------------------------
 */
#define mVlNew( )	AsnListNew(sizeof(int))
#define mVlPut(l,v)	*((int *)AsnListAppend(l))=(int)v
#define mVlFree(l)	(void) FreeVarBindList( (VarBindList *) l )
#define	mVlPrint(l)	(void) PrintVarBindList( stdout, (VarBindList *) l, 0 );\
					fflush(stdout)

#endif 

