/* -----------------------------------------------------------------------
 *
 * Project:		UT-SNMP
 *
 * Filename:    sim.h
 *
 * Date:        July 1994
 *
 * Purpose:		header file for sim module.
 *
 * Author(s):	Roberto Post, Vincent - General Motors - Berkhout, 
 *				Frederic P. H. van Hengstum,
 *
 * -----------------------------------------------------------------------
 */

#ifndef SIM_H
#define SIM_H

#include <snmpTypes.h>


/* 
 *
 *	SIM INTERFACE TYPE DEFINITIONS.
 */

/*
 *	IpcADDRESS type
 */
#define IADDRESSMAX (256)
typedef	char IADDRESS[IADDRESSMAX]; 

/*
 *	IpcMESSAGE type
 */
#define IMESSAGE char*

/*
 *	SIM INTERFACE CALLS
 */
EXTERN STATUS simOpen( IADDRESS localAddress, IADDRESS remoteAddress );
EXTERN STATUS simSend( IADDRESS remoteAddress, IMESSAGE message );
EXTERN STATUS simReceive( IADDRESS remoteAddress, IMESSAGE message );
EXTERN STATUS simClose( IADDRESS remoteAddress );



/* 
 *
 *	SIM UPF TYPE DEFINITIONS
 */
/*
 *	IpcHANDLE type
 */
typedef int		IHANDLE;

/*
 *	IpcDOMAIN type
 */
typedef enum {
    TCP_DOMAIN=0, UDP_DOMAIN=1, PIPE_DOMAIN=2, SHM_DOMAIN=3, MSGP_DOMAIN=4
} IDOMAIN;

/*
 *	IpcCONNECTTABLE type
 */
#define ICONNECTMAX 32
typedef struct {
	IADDRESS remoteAddress;
	IHANDLE dataHandle;
	IDOMAIN dataDomain;
} ICONNECTTABLE[ICONNECTMAX];

/*
 *	IpcLISTENTABLE type
 */
typedef struct {
	IHANDLE listenHandle;
} ILISTENTABLE[MSGP_DOMAIN+1];


/* 
 *
 *	SIM CL CALLS
 */
EXTERN STATUS lpfCheckInputs( IDOMAIN domain, IHANDLE *allHandles, 
			 				  IHANDLE *activeHandles );
EXTERN STATUS lpfPassiveOpen( IDOMAIN domain, IADDRESS localAddress, 
							  IHANDLE *handle );
EXTERN STATUS lpfActiveOpen( IDOMAIN domain, IADDRESS l, IADDRESS r, IHANDLE *handle );
EXTERN STATUS lpfAcceptOpen( IDOMAIN domain, IHANDLE *connectHandle, 
							 IADDRESS address, IHANDLE *handle );
EXTERN STATUS lpfSend( IDOMAIN domain, IHANDLE handle, IMESSAGE message );
EXTERN STATUS lpfReceive( IDOMAIN domain, IHANDLE *handle, IMESSAGE msg );
EXTERN STATUS lpfClose( IDOMAIN domain, IHANDLE handle );


#endif /* ! SIM_H */

/* -----------------------------------------------------------------------
 */
