/* --------------------------------------------------------------------------
 * Project:     SNMP
 * Filename:    snmpInitAgent.c 
 * Date:        April 1994  
 * Purpose:     initialisation *.conf cmu files. 
 * Author( s):  Edward Veluwenkamp & Michel van Ooijen 
 * --------------------------------------------------------------------------
*/

#define MAIN /* This object module is the Main module */

/* -----------------------------------------------------------------------
 *  include files
 * -----------------------------------------------------------------------
 */
#include <snmpInitAgent.h>        /* New header files */
#include <snmpInitAgentUser.h>    /* User interface functions */
#include <snmpInitAgentSys.h>     /* System functions */



/* -----------------------------------------------------------------------
 * Function : InitViewRecord
 * Arguments: view initial pointer to viewentry (IN/OUT)
 * Purpose  : This function initialises a view record.
 * Remarks  :
 * -----------------------------------------------------------------------
 */
void InitViewRecord(view)
  ViewEntry *view;
{
  view->viewIndex = 1;          /* First record in view.conf file */
  strcpy(view->viewSubtree,""); /* Variable empty */
  strcpy(view->viewMask,"");
  strcpy(view->viewType,"");
}



/* -----------------------------------------------------------------------
 * Function : AddViewRecordCmu 
 * Arguments: fp, filepointer viewfile (IN/OUT), 
	      view, initial pointer to viewentry  (IN/OUT)  
 * Purpose  : This function adds a record to cmu view file 
 * Remarks  :
 * -----------------------------------------------------------------------
 */
void InitAddViewRecordToCmu(fp,view)
  FILE *fp;
  ViewEntry view; 
  { /* Store view record as cmu format with tabs*/
  fprintf (fp,"\n");
  fprintf (fp,"%-3d\t%-20s\t%-10s\t%s\n",view.viewIndex,view.viewSubtree,view.viewMask,view.viewType);
  fprintf( stdout, "v" ); /* view record stored */
  fflush( stdout );
  usleep( 200000 ); /* 0.2 sec delay */
  }


/* -----------------------------------------------------------------------
 * Function : InitChangeViewRecord 
 * Arguments: KindMessage,    (IN),
	      fp, FILE pointer (IN/OUT),
	      view, initial pointer to viewentry  (IN/OUT) 
 * Purpose  : This function change a view record and store this record in 
	      the view cmu file. 
 * Remarks  : 
 * -----------------------------------------------------------------------
 */
void InitChangeViewRecords(KindMessage,fp,view)
  int KindMessage;
  FILE *fp;
  ViewEntry *view;
  {
  char viewclose = 0; /* Local var */
  do
    {
    if (KindMessage == 0)
      switch(viewclose) /* Three times store a view subtree in view file */
	{
	case 0:strcpy(view->viewSubtree,".1.3.6.1.2.1.1");     /* system */
	       break;
	case 1:strcpy(view->viewSubtree,".1.3.6.1.6.3.1.1.1"); /* snmpStats */
	       break;
	case 2:strcpy(view->viewSubtree,".1.3.6.1.6.3.3.2.1"); /* snmpParties */
        }
    else 
      {
      strcpy(view->viewSubtree,".1.3.6.1"); /* Internet */
      viewclose = 2; /* End of while loop */
      }
    strcpy(view->viewMask,"included"); /* Initialisation viewMask */
    strcpy(view->viewType,"Null");
    InitAddViewRecordToCmu(fp, *view); /* Add new record */  
    viewclose++; /* counter loop */
    } while (viewclose < 3); 
    view->viewIndex++; /* Counter index from view cmu records */
  }


/* -----------------------------------------------------------------------
 * Function : InitAddViewcmufile 
 * Arguments: KindMessage, initial int  (IN)
              view, initial pointer to viewentry  (IN/OUT)
              hostnameA[], initial char arr  (IN)
 * Purpose  : This function adds a cmu record to the View file table  
 * Remarks  : Exit -1 if file unable to open for saving 
 * -----------------------------------------------------------------------
 */
void InitAddViewcmufile(KindMessage,view,hostnameA)
 int KindMessage;
 ViewEntry *view;  
 char *hostnameA;
{
 
 FILE *fp; /* Local filepointer to the cmu view file */
 char filename[FILENAMELEN]=""; /* Temp. arr. to store a filename */
 if( getenv( "SNMP_PARTYMIB_PATH" ) != (char *) NULL )
	strcat(filename,getenv( "SNMP_PARTYMIB_PATH"));
 strcat(filename,hostnameA);    
 strcat(filename,Viewfile);     /* Store viewname in temp. arr */
 if ((fp=fopen(filename,"a")) == (FILE *) NULL) /* Open the view file for append */
     {
     fprintf (stdout,"\t\tERROR: unable to open %s for saving.\n", filename );
     exit(-1);
     }

 InitChangeViewRecords(KindMessage,fp, view); /* Change and adds a record to a cmu view file */
 fclose (fp); /* Close and flush the cmu view file */
 }


/* -----------------------------------------------------------------------
 * Function:  	InitAclRecord
 * Arguments: 	acl, initial pointer to aclentry (IN/OUT)
 * Purpose:   	This function initialises an acl record.
 * Remarks:
 * -----------------------------------------------------------------------
 */
void InitAclRecord(acl)
  AclEntry *acl;
{
  strcpy(acl->aclTarget,""); /* Initialises acl record */
  strcpy(acl->aclSubject,""); /* null char at first position */
  strcpy(acl->aclResources,"");
  strcpy(acl->aclPrivileges,"");
}


/* -----------------------------------------------------------------------
 * Function : InitAddAclRecordCmu 
 * Arguments: fp,  initial filepointer (IN/OUT)
	      acl, initial pointer to aclentry (IN/OUT) 
 * Purpose  : This function adds a record to a cmu acl file 
 * Remarks  : 
 * -----------------------------------------------------------------------
 */
void InitAddAclRecordToCmu(fp,acl)
  FILE *fp;
  AclEntry acl; 
  { /* Store acl record as cmu format */

  fprintf (fp,"\n"); /* New line */
  fprintf (fp,"%-36s\t%-36s\t%-26s\t%s\n",acl.aclTarget,acl.aclSubject,acl.aclResources,acl.aclPrivileges);/* add record */
  fprintf( stdout, "a" ); /* Acl record stored */
  fflush( stdout );
  usleep( 200000 ); /* 0.2 sec delay */
  }


/* -----------------------------------------------------------------------
 * Function  : InitChangeAclRecord 
 * Arguments : KindMessage, initial int (IN) 
               fp, filepointer (IN/OUT)
	       acl, initial pointer to aclentry  (IN/OUT) 
               party1, initial pointer to partyentry agent (IN/OUT)
               party2, initial pointer to partyentry manager (IN/OUT)
 * Purpose   : This function change the acl records in the acl file
 * Remarks   :
 * -----------------------------------------------------------------------
 */
void InitChangeAclRecords(KindMessage,fp,acl,party1,party2,context)
  int KindMessage;
  FILE *fp;
  AclEntry *acl;  
  PartyEntry *party1,*party2;
  ContextEntry *context;
  {  
    strcpy(acl->aclTarget,party1->FriendbyName); /* acl initialisation agent name */
    strcpy(acl->aclSubject,party2->FriendbyName); /* acl initialisation manager name */
    strcpy(acl->aclResources,context->contextName); /* Context name in acl record */
    switch(KindMessage)
      {
      case 0:strcpy(acl->aclPrivileges,"gnb");/* g = get, n = getnext, b = Bulk */
	     break;
      case 1:;
      case 2:strcpy(acl->aclPrivileges,"gnsbi");/* g = get, n = getnext, s = set, b = Bulk i = Inform*/
      }
    InitAddAclRecordToCmu(fp, *acl); /* add record */

    strcpy(acl->aclTarget,party2->FriendbyName); /* second party */ 
    strcpy(acl->aclSubject,party1->FriendbyName);
    strcpy(acl->aclResources,context->contextName);
    switch(KindMessage)
      {
      case 0:strcpy(acl->aclPrivileges,"ru"); /* r = Get response, u = Trap  */
	     break;
      case 1:;
      case 2:strcpy(acl->aclPrivileges,"r"); /* r = Get response */
      }
    InitAddAclRecordToCmu(fp, *acl); /* add change record */
  }


/* -----------------------------------------------------------------------
 * Function : InitAddAclcmufile 
 * Arguments: KindMessage, initial int (IN)
              acl, initial pointer to AclEntry acl (IN/OUT)
              party1, initial pointer to partyentry agent (IN/OUT)
              party2, initial pointer to partyentry manager (IN/OUT)
	      hostnameA[], initial char arr (IN/OUT)
 * Purpose  : This function adds a acl record in the cmu Acl table  
 * Remarks  :  
 * -----------------------------------------------------------------------
 */
void InitAddAclcmufile(KindMessage,acl,party1,party2,context,hostnameA)
  int KindMessage;
  AclEntry *acl;  /* Local variable */
  PartyEntry *party1,*party2;
  ContextEntry *context;
  char *hostnameA;
 {
 FILE *fp; /* local filepointer */
 char filename[FILENAMELEN]="";                      /* temp. arr. use to store a filename, initial empty */
 if( getenv( "SNMP_PARTYMIB_PATH" ) != (char *) NULL )
	strcat(filename,getenv( "SNMP_PARTYMIB_PATH"));
 strcat(filename,hostnameA);
 strcat(filename,Aclfile); /* Acl filename */
 if ((fp=fopen(filename,"a")) == (FILE *) NULL) /* file open for append */
     {
     fprintf (stderr,"\t\tERROR: unable to %s for saving.\n", filename );
     exit(-1);
     }
 InitChangeAclRecords(KindMessage,fp, acl, party1, party2, context);
 fclose (fp);
 }


/* -----------------------------------------------------------------------
 * Function :  	InitContextRecord
 * Arguments: 	context, initial pointer to contextentry (IN/OUT) 
 * Purpose  :  	This function initialises an context record.
 * Remarks  :
 * -----------------------------------------------------------------------
 */
void InitContextRecord( ContextEntry *context )
  {
  strcpy(context->IPaddress,"");
  strcpy(context->contextName,"");
  strcpy(context->contextIdentity,ContextIdentityStand);
  context->contextIndex = 1;
  context->contextLocal = 0;
  context->contextViewIndex = 1;
  strcpy(context->contextLocalEntity,"Null");
  strcpy(context->contextLocalTime,"currentTime");
  strcpy(context->contextProxyDstParty,"0");
  strcpy(context->contextProxySrcParty,"0");
  strcpy(context->contextProxyContext,".0.0");
  }


/* -----------------------------------------------------------------------
 * Function : InitAddContextRecordCmu 
 * Arguments: fp, filepointer (IN/OUT) 
	      context, initial pointer to contextentry (IN/OUT) 
 * Purpose  : This function adds a record to cmu context file 
 * Remarks  :
 * -----------------------------------------------------------------------
 */
void InitAddContextRecordToCmu(fp,context)
  FILE *fp;
  ContextEntry context; 
  { /* Store context record as cmu format */

  if (strlen(context.contextLocalEntity) == 0)
    strcpy(context.contextLocalEntity,"Null");  

  fprintf (fp,"\n");
  fprintf (fp,"%-20s\t%s.%d\n",context.contextName, context.contextIdentity,context.contextIndex);
  fprintf (fp,"%-20d\t%-36s\t%s\n",context.contextViewIndex,context.contextLocalEntity,context.contextLocalTime);
  fprintf (fp,"%-20s\t%-36s\t%s\n",context.contextProxyDstParty, context.contextProxySrcParty, context.contextProxyContext);
  fprintf( stdout, "c" );
  fflush( stdout );
  usleep(200000); /* 0.2 sec */
  }


/* -----------------------------------------------------------------------
 * Function : InitChangeContextRecord 
 * Arguments: KindMessage (IN)
 *            fp, filepointer (IN/OUT)
 *            party, initial pointer to PartyEntry (IN/OUT) 
 *            context, initial pointer to contextentry (IN/OUT)
 *            view, ViewEntry (IN/OUT)
 *            hostnameA, initial host name from the agent (IN)
 * Purpose  : This function change the context record and add this 
	      record in the context file
 * Remarks  :
 * -----------------------------------------------------------------------
 */
void InitChangeContextRecords(KindMessage,fp,party, context,view,hostnameA)
  int KindMessage;
  FILE *fp;
  PartyEntry *party;
  ContextEntry *context;
  ViewEntry *view;
  char *hostnameA;
  {
  UINcontextuser(KindMessage,context->contextName,hostnameA);
  strcpy(context->IPaddress,party->IPaddress); /* IP address in the context equal to the IP address in the party */
  strcpy(context->contextIdentity,ContextIdentityStand);  
  strcat(context->contextIdentity,context->IPaddress);
  context->contextViewIndex = view->viewIndex - 1; /* correction with -1 */
  InitAddContextRecordToCmu(fp, *context); /* Add context record */
  context->contextIndex++; /* Counter context identifier */
  }



/* -----------------------------------------------------------------------
 * Function: InitAddContextcmufile 
 * Arguments: KindMessage (IN)
 *            party, initial pointer to partyentry (IN/OUT) 
 *            context, initial pointer to contextentry (IN/OUT)
 *            view, ViewEntry (IN/OUT)
 *            hostnameA, initial host name from the agent (IN)
 * Purpose:  This function adds a cmu context record to the context table  
 * Remarks:
 * -----------------------------------------------------------------------
 */
void InitAddContextcmufile(KindMessage,party,context,view,hostnameA)
  int KindMessage;
  PartyEntry *party;
  ContextEntry *context; 
  ViewEntry *view;
  char *hostnameA;
{
 FILE *fp;
 char filename[FILENAMELEN]="";
 if( getenv( "SNMP_PARTYMIB_PATH" ) != (char *) NULL )
	strcat(filename,getenv( "SNMP_PARTYMIB_PATH"));
 strcat(filename,hostnameA);
 strcat(filename,Contextfile);

 if ((fp=fopen(filename,"a")) == (FILE *) NULL)
     {
     fprintf (stderr,"\t\tERROR: unable to open %s for saving.\n",filename );
     exit(-1);
     }

 InitChangeContextRecords(KindMessage,fp,party, context, view, hostnameA);
 fclose (fp);
}


/* -----------------------------------------------------------------------
 * Function:  InitAssignPartyRecord
 * Arguments: KindMessage, initial int (IN)
 *            party, initial pointer to partyentry (IN/OUT) 
 * Purpose:   This function change the partyAuthProtocol and partyAuthPrivate
 *            in the party record
 * Remarks:
 * -----------------------------------------------------------------------
 */
void InitAssignPartyRecord(KindMessage,party)
  int KindMessage;
  PartyEntry *party;
{
   switch(KindMessage)
     {
     case 0:strcpy(party->partyAuthProtocol,"noAuth");
            strcpy(party->partyAuthPrivate,"00000000000000000000000000000000");
	    break;
     case 1:;
     case 2:strcpy(party->partyAuthProtocol,"snmpv2md5Auth");
            SINgetrandomkey(party->partyAuthPrivate); /* random authpriv key */
     }
   switch(KindMessage)
     {
     case 0:;
     case 1:strcpy(party->partyPrivProtocol,"noPriv"); 
            strcpy(party->partyPrivPrivate,"00000000000000000000000000000000");
	    break;
     case 2:strcpy(party->partyPrivProtocol,"desPriv");
	    sleep( 1 );
            SINgetrandomkey(party->partyPrivPrivate); /* random privprivate key */
     }
}



/* -----------------------------------------------------------------------
 * Function:  	InitPartyRecord
 * Arguments: 	party, initial pointer to partyentry (IN/OUT) 
 * Purpose:   	This function initialises an party record.
 * Remarks:
 * -----------------------------------------------------------------------
 */
void InitPartyRecord(party)
  PartyEntry *party;
{
   strcpy(party->FriendbyName,"");  /* Initialisation party record */
   strcpy(party->partyIdentity,"");
   strcpy(party->IPaddress,"");
   strcpy(party->partyTAddress,"");
   strcpy(party->partyAuthProtocol,"");
   strcpy(party->partyAuthPrivate,"");
   strcpy(party->partyAuthPublic,"");
   strcpy(party->partyPrivProtocol,"");
   strcpy(party->partyPrivPrivate,"");
   strcpy(party->partyPrivProtocol,"");
   strcpy(party->partyPrivPublic,"");
   strcpy(party->partyMaxMessageSize,"0");      /* Default */
   strcpy(party->partyTDomain,"snmpUDPDomain"); /* Default */
   strcpy(party->partyAuthClock,"00000000");    /* Default */
   strcpy(party->partyAuthLifeTime,"300");      /* Default */
   party->partyLocal = 0; 		       /* Default */	
}


/* -----------------------------------------------------------------------
 * Function  : InitAddPartyRecordCmu 
 * Arguments : fp ,party (IN/OUT) 
 * Purpose   : This function adds a record in the cmu party file 
 * Remarks   :
 * -----------------------------------------------------------------------
 */
void InitAddPartyRecordToCmu(fp,party)
  FILE *fp;
  PartyEntry party; 
  { /* Store party record as cmu format */
  int privPublicLength = 0;            /* Local variable */
  int authPublicLength = 0;
  char privPublicLengthstr[ARLEN] = "";
  char authPublicLengthstr[ARLEN] = "";

  privPublicLength = strlen(party.partyPrivPublic); /* calculate length PrivPublic */
  if (privPublicLength == 0)
    strcpy(privPublicLengthstr,"Null"); /* length == 0 */
  else sprintf(privPublicLengthstr,"%d",privPublicLength); /* convert integer to string */

  authPublicLength = strlen(party.partyAuthPublic); /* calculate length AuthPublic */
  if (authPublicLength == 0)
    strcpy(authPublicLengthstr,"Null"); /* length == 0 */ 
  else sprintf(authPublicLengthstr,"%d",authPublicLength); /* convert integer to string */

  fprintf (fp,"\n\n\n");
  fprintf (fp,"%-36s\t%s.%d\n",party.FriendbyName, party.partyIdentity, party.partyIndex); /* Party record */
  fprintf (fp,"%-36s\t%-36s\t%s\n",party.partyTDomain,party.IPaddress,party.partyTAddress);
  fprintf (fp,"%-36s\t%s\n",party.partyAuthProtocol,party.partyPrivProtocol);
  fprintf (fp,"%-36s\t%s\n",party.partyAuthLifeTime,party.partyMaxMessageSize);
  fprintf (fp,"%-36s\n",party.partyAuthClock);
  fprintf (fp,"%-36s\t%-10s\t%s\n",party.partyAuthPrivate,authPublicLengthstr,party.partyAuthPublic);
  fprintf (fp,"%-36s\t%-10s\t%s\n",party.partyPrivPrivate,privPublicLengthstr,party.partyPrivPublic);
  fprintf( stdout, "p" );
  fflush( stdout );
  usleep(200000); /* 0.2 sec delay */
  }


/* -----------------------------------------------------------------------
 * Function: 	InitEmptyFile 
 * Arguments: 	IniFile, the filename from the init CMU headerfile
 *		FileName, the filename in which the CMU header is stored  
 *		hostnameA, the hostname 
 * Purpose:  	This function adds the record in cmu party file 
 * Remarks:
 * -----------------------------------------------------------------------
 */
void InitEmptyfile(IniFile,FileName,hostnameA)
 char *IniFile;
 char *FileName;
 char *hostnameA;
{
 FILE *fp1;  /* Local filepointer append file */
 FILE *finit; /* Local filepointer read file */
 int buffer; /* Store read int in Temp. integer */
 char filename[FILENAMELEN] = ""; /* store filename */
 /* strcpy(filename,CMU_PATH);
  */
 if( getenv( "SNMP_PARTYMIB_PATH" ) != (char *) NULL )
	strcat( filename, getenv( "SNMP_PARTYMIB_PATH" ) );  
 strcat(filename,hostnameA);
 strcat(filename,FileName);
 if ((fp1=fopen(filename,"w")) == (FILE *) NULL) {
     fprintf( stdout,"\t\tERROR: unable to open %s for writing.\n", filename );
     exit(-1);
 } else {
	fprintf( stdout, "\tOpening %s. \n ", filename );
 }
 /* strcpy(filename,CMU_PATH);
  */
 strcpy(filename,""); /* make string empty */
 if( getenv( "SNMP_PARTYMIB_INITPATH" ) != (char *) NULL )
	strcat( filename, getenv( "SNMP_PARTYMIB_INITPATH" ) );  
 strcat(filename,IniFile);
 if( ( finit=fopen(filename,"r") ) == (FILE *) NULL ) {
     fprintf( stdout,"\t\tERROR: unable to open %s for reading.\n", filename );
     exit(-1);
 }
 while ((buffer=getc(finit)) != EOF) { /* read the input file and append it in the output file */
   putc(buffer, fp1);
 }
 fclose(finit); /* close read init file */ 
 fclose(fp1);   /* close write file */
}



/* -----------------------------------------------------------------------
 * Function : InitAddPartycmufile
 * Arguments: none 
 * Purpose  : This function adds a records in the cmu party, context, 
	      view and acl table 
 * Remarks  :
 * -----------------------------------------------------------------------
 */
void InitAddcmufiles(void)
{
   FILE *fp;  /* local filepointer */
   char hostnameM[ARLEN]; /* Temp arr. for the Name of the Manager */
   char hostnameA[ARLEN]; /* Temp arr. for the Name of the Agent */
   char filename[FILENAMELEN] = ""; /* Temp. arr for. the name for the filename */
   int i; /* local int counter */

   PartyEntry party1;  /* First party */
   PartyEntry party2;  /* Second party */
   ContextEntry context; /* Context table */ 
   ViewEntry view;  /* viewtable */
   AclEntry acl; /* Acl table */
   int  KindMessage= 0; /* NoAuth = 0, Auth = 1, md5 = 2 */


   InitContextRecord(&context); /* Init Context record */
   InitViewRecord(&view); /* Init view record */
   InitAclRecord(&acl); /* Init acl record */
   InitPartyRecord(&party1); /* Agent party */
   InitPartyRecord(&party2); /* Manager party */

   UINintro( hostnameM, party2.IPaddress, hostnameA, party1.IPaddress,
	     party1.partyMaxMessageSize, party2.partyTAddress , party1.partyTAddress); /* Communication with the user */

   fprintf( stdout, "\n Status:\n" ); /* message user */
   strcpy(party2.partyMaxMessageSize,party1.partyMaxMessageSize); /* manager and agent equal */

   strcpy(party1.partyIdentity,PartyIdentityStand);  /* standard OID */
   strcat(party1.partyIdentity,party1.IPaddress); /* strcat IP address */
   strcpy(party2.partyIdentity,PartyIdentityStand); /* standard OID */
   strcat(party2.partyIdentity,party1.IPaddress); /* strcat IP address */
   
   InitEmptyfile(PartyIniFile,Partyfile,hostnameA); /* Init CMU Header in 'hostname'_party.conf file */ 
   InitEmptyfile(ContextIniFile,Contextfile,hostnameA); /* Init CMU Header in 'hostname'_context.conf file */
   InitEmptyfile(AclIniFile,Aclfile,hostnameA); /* Init Header in 'hostname'_acl.conf file */
   InitEmptyfile(ViewIniFile,Viewfile,hostnameA); /* Init Header in 'hostname'_view.conf file */

   /* strcpy(filename,CMU_PATH); */
   strcpy(filename,""); /* empty */
   if( getenv( "SNMP_PARTYMIB_PATH" ) != (char *) NULL )
	strcat( filename, getenv( "SNMP_PARTYMIB_PATH" ) );  

   strcat(filename,hostnameA);
   strcat(filename,Partyfile);
   if ((fp=fopen(filename,"a+")) == (FILE *) NULL) { /* opening for read/append */
     fprintf (stderr,"\tERROR: unable to open %s for saving.\n", filename );
     exit(-1);
   }
   party1.partyIndex = 1; /* Init counter partie 1 */			
   party2.partyIndex = 2; /* Init counter partie 2 */	
   do {
     InitAssignPartyRecord(KindMessage,&party1); /* Agent party */
     InitAssignPartyRecord(KindMessage,&party2); /* Manager party */
     strcpy(party2.partyAuthPrivate,party1.partyAuthPrivate);
     strcpy(party2.partyPrivPrivate,party1.partyPrivPrivate);

     if (!UINpartyuser(KindMessage,party2.FriendbyName,hostnameA,party1.FriendbyName))
       {
       InitAddPartyRecordToCmu(fp, party1); /* Store party Agent in the cmu party file */
       InitAddPartyRecordToCmu(fp, party2); /* Store party Manager in the cmu party file */
       if (KindMessage < 2)
	 {
         InitAddViewcmufile(KindMessage,&view,hostnameA);
	 InitAddContextcmufile(KindMessage,&party1,&context,&view,hostnameA); 
	 }
       InitAddAclcmufile(KindMessage,&acl,&party1,&party2,&context,hostnameA); 
       party1.partyIndex = party1.partyIndex + 2; /* Counter next parties */			
       party2.partyIndex = party2.partyIndex + 2;
       }
      KindMessage ++; 
   } while (KindMessage < 3); /* six parties max input */
   fclose (fp);
   fprintf( stdout, " records.\n\tClosed all files.\n\n " );
   for( i = 0; i < 72; i++ ) { /* output terminal */
	fprintf( stdout, "*" );
   }
   fprintf( stdout, "\n \n" );
}


/* -----------------------------------------------------------------------
 * Function : main 
 * Arguments: none 
 * Purpose  : This main function init. the party, context, view and acl 
	      cmu *.conf files.
 * Remarks  : 
 * -----------------------------------------------------------------------
 */
void main()
{
 InitAddcmufiles();
}

