/* -----------------------------------------------------------------------
 *
 * Project:	SNMPV2
 *
 * Filename:    spmPartyINT.c
 *
 * Date:        April 1994
 *
 * Purpose:     This file contains the partymib API to the meta-manager.
 *
 * Author( s):  Koert W.G. Flapper
 *
 * -----------------------------------------------------------------------
 */

#include <spmPartyMIB.h>

AsnOcts  DefTAddress             = { 6, "\0\0\0\0\0\0" };
AsnOcts  EmptyOctetstring        = { 0, "" };
AsnOid   DefPCF                  = { 2, "\0\0" };

/****************************************************************************************
 * INITIALIZATION PART TO INSERT IN GENERIC MIB IMPLEMENTATION WITH ROUTINE "MibInit" 	*
 ****************************************************************************************/

int	partymsize = 14, partytsize = 4;
/*	OIDs are in octal representation */
struct MibRecord partymibtab[] = {
	{ "partyMIB", { 6, "\53\6\1\6\3\3" }, AGGREGATE, FALSE, FALSE, 0, 0, 0},
	{ "partyMIBObjects", { 7, "\53\6\1\6\3\3\2" }, AGGREGATE, FALSE, FALSE, 0, 0, 0},
	{ "snmpParties", { 8, "\53\6\1\6\3\3\2\1" }, AGGREGATE, FALSE, FALSE, 0, 0, 0},
	{ "partyTable", { 9, "\53\6\1\6\3\3\2\1\1" }, AGGREGATE, FALSE, FALSE, 0, 0, 0},
	{ "partyEntry", { 10, "\53\6\1\6\3\3\2\1\1\1" }, TABLE, FALSE, FALSE, MibTableRequest,
(AsnInt) &partytabtab[0], 0},
	{ "snmpContexts", { 8, "\53\6\1\6\3\3\2\2" }, AGGREGATE, FALSE, FALSE, 0, 0, 0},
	{ "contextTable", { 9, "\53\6\1\6\3\3\2\2\1" }, AGGREGATE, FALSE, FALSE, 0, 0, 0},
	{ "contextEntry", { 10, "\53\6\1\6\3\3\2\2\1\1" }, TABLE, FALSE, FALSE, MibTableRequest,
(AsnInt) &partytabtab[1], 0},
	{ "snmpAccess", { 8, "\53\6\1\6\3\3\2\3" }, AGGREGATE, FALSE, FALSE, 0, 0, 0},
	{ "aclTable", { 9, "\53\6\1\6\3\3\2\3\1" }, AGGREGATE, FALSE, FALSE, 0, 0, 0},
	{ "aclEntry", { 10, "\53\6\1\6\3\3\2\3\1\1" }, TABLE, FALSE, FALSE, MibTableRequest,
(AsnInt) &partytabtab[2], 0},
	{ "snmpViews", { 8, "\53\6\1\6\3\3\2\4" }, AGGREGATE, FALSE, FALSE, 0, 0, 0},
	{ "viewTable", { 9, "\53\6\1\6\3\3\2\4\1" }, AGGREGATE, FALSE, FALSE, 0, 0, 0},
	{ "viewEntry", { 10, "\53\6\1\6\3\3\2\4\1\1" }, TABLE, FALSE, FALSE, MibTableRequest,
(AsnInt) &partytabtab[3], 0}
};

struct TabRecord partytabtab[] = {
	{ PartyTableGet, PartyTableGetFirst, PartyTableGetNext, PartyTableSet, 0},
	{ ContextTableGet, ContextTableGetFirst, ContextTableGetNext, ContextTableSet, 0},
	{ AclTableGet, AclTableGetFirst, AclTableGetNext, AclTableSet, 0},
	{ ViewTableGet, ViewTableGetFirst, ViewTableGetNext, ViewTableSet, 0}
};

/****************************************************************************************
 *											*
 *	IMPLEMENTATION OF TABLE-SPECIFIC ROUTINES FOR THE MANAGEMENT OF THE PARTYMIB 	*
 *											*
 ****************************************************************************************/


/*
 * -----------------------------------------------------------------------
 * Function:    PartyTableGet
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     Executes a GET operation on the managed object specified by
 *		oid and stores the result in the variable binding vbc
 *
 * Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * -----------------------------------------------------------------------
 */
	STATUS
PartyTableGet( AsnOid *oid, struct VarBindChoice *vbc)
{
	PField			field;
	struct PartyEntry	*pp;

	if ( PartyMatch(oid, &field, &pp) != OK) {
		free( vbc->a.value);
		return(FAILED);		/* no such object/instance */
	} else
		return( PartyTableFetch( field, pp, vbc));
}

/*
 * -----------------------------------------------------------------------
 * Function:    PartyMatch
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		PField			*field	(OUT)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     This routine searches the partyTable for an entry which
 *		matches oid. The found entry and field are returned
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS
PartyMatch( AsnOid *oid, PField *field, struct PartyEntry **pp)
{
	int	oidi, loc;
	Party	*oidp;
	struct Hashnode	*phn;

	oidi = PARTYPREFIX;
	if (oid->octetLen <= oidi) {
		errno = NOSUCHOBJECT;
		return(FAILED);
	}
	*field = (PField) *(char *)(oid->octs + oidi++);
	if (( *field < PIDENTITY) || (*field > PSTATUS)) {
		errno = NOSUCHOBJECT;
		return(FAILED);
	}

	oidp = pNewOid(oid->octetLen - oidi);
	if ((oid->octetLen > (oidi+1)) &&((oid->octs[oidi] <= 2) && ( oid->octs[oidi+1] < 40))) {
		oidp->octetLen = (oid->octetLen - oidi) - 1;
		oidp->octs[0] = (oid->octs[oidi]*40) + oid->octs[oidi+1];
		memcpy( &oidp->octs[1], (oid->octs+(oidi+2)), oidp->octetLen-1);
	} else {
		oidp->octetLen = oid->octetLen - oidi;
		memcpy( oidp->octs, (oid->octs + oidi), oidp->octetLen);
	}

	loc = hashoid(oidp, psize, pradix);
	phn = phashtab[loc];
	while (phn != NULL) {
		if ( CMPOID( oidp, ((PartyEntry *)phn->pentry)->partyIdentity) == EQUAL) {
			*pp = phn->pentry;
			FreeOid(oidp);
			return(OK);
		}
		phn = phn->pnext;
	}
	FreeOid(oidp);
	errno = NOSUCHINSTANCE;
	return(FAILED);
}


/*
 * -----------------------------------------------------------------------
 * Function:    PartyTableFetch
 *
 * Arguments:   PField			field	(IN)
 *		struct PartyEntry	*pp	(IN)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     Fetches the columnar object instance specified by field from
 *		the partyEntry specified by pp and stores the result in vbc
 *
 * Remarks:	Allocation of memory for values with variable size is done
 *		inside this routine
 *
 * -----------------------------------------------------------------------
 */
	STATUS
PartyTableFetch( PField field, struct PartyEntry *pp, struct VarBindChoice *vbc)
{
	ObjectSyntax	*object;

	object = vbc->a.value;
	switch(field) {

	case PIDENTITY: /* not-accessible */
		vbc->choiceId = VARBINDCHOICE_NOSUCHINSTANCE;
		return(FAILED);

	case PINDEX: /* read-only */
		object->choiceId = OBJECTSYNTAX_INTEGER_VALUE;
		object->a.integer_value = pp->partyIndex;
		return(OK);

	case PTDOMAIN: /* read-create */
		object->choiceId = OBJECTSYNTAX_OBJECTID_VALUE;
		if (pp->partyTDomain) {
			object->a.objectID_value = pNewOid( pp->partyTDomain->octetLen);			
			object->a.objectID_value->octetLen = pp->partyTDomain->octetLen;
			memcpy( object->a.objectID_value->octs, pp->partyTDomain->octs,
object->a.objectID_value->octetLen);
		} else	return(FAILED);
		return(OK);

	case PTADDRESS: /* read-create */
		object->choiceId = OBJECTSYNTAX_STRING_VALUE;
		if (pp->partyTAddress) {
			if ((object->a.string_value = pNewOct(pp->partyTAddress->octetLen)) == NULL)
				return(ERROR);
			object->a.string_value->octetLen = pp->partyTAddress->octetLen;
			memcpy( object->a.string_value->octs, pp->partyTAddress->octs,
object->a.string_value->octetLen);
		} else	return(FAILED);
		return(OK);

	case PMAXMESSAGESIZE: /* read-create */
		object->choiceId = OBJECTSYNTAX_INTEGER_VALUE;
		if (pp->partyMaxMessageSize)
			object->a.integer_value = pp->partyMaxMessageSize;
		else	return(FAILED);
		return(OK);

	case PLOCAL: /* read-create */
		object->choiceId = OBJECTSYNTAX_INTEGER_VALUE;
		if (pp->partyLocal)
			object->a.integer_value = pp->partyLocal;
		else	return(FAILED);
		return(OK);

	case PAUTHPROTOCOL: /* read-create */
		object->choiceId = OBJECTSYNTAX_OBJECTID_VALUE;
		if (pp->partyAuthProtocol) {
			if ((object->a.objectID_value = pNewOct(pp->partyAuthProtocol->octetLen)) ==
NULL)
				return(ERROR);
			object->a.objectID_value->octetLen = pp->partyAuthProtocol->octetLen;
			memcpy( object->a.objectID_value->octs, pp->partyAuthProtocol->octs,
pp->partyAuthProtocol->octetLen);
		} else	return(FAILED);
		return(OK);
		
	case PAUTHCLOCK: /* read-create */
		object->choiceId = OBJECTSYNTAX_UNSIGNED_INTEGER_VALUE;
		if (pp->partyAuthClock)
			object->a.unsigned_integer_value = pp->partyAuthClock;
		else	return(FAILED);
		return(OK);

	case PAUTHPRIVATE: /* read-create; when read, the value of this variable is the zero length
OCTET STRING */
		object->choiceId = OBJECTSYNTAX_STRING_VALUE;
		if ((object->a.string_value = pNewOct( EmptyOctetstring.octetLen)) == NULL)
			return(ERROR);
		object->a.string_value->octetLen = EmptyOctetstring.octetLen;
		memcpy( object->a.string_value->octs, &EmptyOctetstring.octs,
object->a.string_value->octetLen);
		return(OK);

	case PAUTHPUBLIC: /* read-create; publically-readable value for the party */
		object->choiceId = OBJECTSYNTAX_STRING_VALUE;
		if (pp->partyAuthPublic) {
			if ((object->a.string_value = pNewOct(pp->partyAuthPublic->octetLen)) == NULL)
				return(ERROR);
			object->a.string_value->octetLen = pp->partyAuthPublic->octetLen;
			memcpy( object->a.string_value->octs, pp->partyAuthPublic->octs,
pp->partyAuthPublic->octetLen);
		} else	return(FAILED);
		return(OK);

	case PAUTHLIFETIME: /* read-create */
		object->choiceId = OBJECTSYNTAX_INTEGER_VALUE;
		if (pp->partyAuthLifetime)
			object->a.integer_value = pp->partyAuthLifetime;
		else	return(FAILED);
		return(OK);

	case PPRIVPROTOCOL: /* read-create */
		object->choiceId = OBJECTSYNTAX_OBJECTID_VALUE;
		if (pp->partyPrivProtocol) {
			if ((object->a.objectID_value = pNewOct(pp->partyPrivProtocol->octetLen)) ==
NULL)
				return(ERROR);
			object->a.objectID_value->octetLen = pp->partyPrivProtocol->octetLen;
			memcpy( object->a.objectID_value->octs, pp->partyPrivProtocol->octs,
pp->partyPrivProtocol->octetLen);
		} else	return(FAILED);
		return(OK);

	case PPRIVPRIVATE: /* read-create; when read, the value of this variable is the zero length
OCTET STRING */
		object->choiceId = OBJECTSYNTAX_STRING_VALUE;
		object->choiceId = OBJECTSYNTAX_STRING_VALUE;
		if ((object->a.string_value = pNewOct( EmptyOctetstring.octetLen)) == NULL)
			return(ERROR);
		object->a.string_value->octetLen = EmptyOctetstring.octetLen;
		memcpy( object->a.string_value->octs, &EmptyOctetstring.octs,
object->a.string_value->octetLen);
		return(OK);

	case PPRIVPUBLIC: /* read-create; publically-readable value for the party */
		object->choiceId = OBJECTSYNTAX_STRING_VALUE;
		if (pp->partyPrivPublic) {
			if ((object->a.string_value = pNewOct(pp->partyPrivPublic->octetLen)) == NULL)
				return(ERROR);
			object->a.string_value->octetLen = pp->partyPrivPublic->octetLen;
			memcpy( object->a.string_value->octs, pp->partyPrivPublic->octs,
pp->partyAuthPublic->octetLen);
		} else	return(FAILED);
		return(OK);

	case PCLONEFROM: /* read-create */
		object->choiceId = OBJECTSYNTAX_OBJECTID_VALUE;
		object->a.objectID_value = pNewOid(DefPCF.octetLen);
		object->a.objectID_value->octetLen = DefPCF.octetLen;
		memcpy( object->a.objectID_value->octs, DefPCF.octs,
object->a.objectID_value->octetLen);
		return(OK);

	case PSTORAGETYPE: /* read-create */
		object->choiceId = OBJECTSYNTAX_INTEGER_VALUE;
		if (pp->partyStorageType)
			object->a.integer_value = pp->partyStorageType;
		else	return(FAILED);
		return(OK);

	case PSTATUS: /* read-create */
		object->choiceId = OBJECTSYNTAX_INTEGER_VALUE;
		if (pp->partyStatus)
			object->a.integer_value = pp->partyStatus;
		else	return(FAILED);
		return(OK);
		
	default:
		vbc->choiceId = VARBINDCHOICE_NOSUCHOBJECT;
		break;
	}
	return(FAILED);
}

/*
 * ------------------------------------------------------------------------
 *
 *	Function:	PartyTableGetFirst
 *
 *	Arguments:	AsnOid			*oid	(IN)
 *			struct MibRecord	*mr	(IN)
 *			struct VarBindChoice	*vbc	(OUT)
 *
 *	Purpose:	Finds the lexicographically first managed object in the
 *			partyTable and stores the result in the variable binding
 *			vbc. If the table is empty, routines are called for the 
 *			lexicographically next object using mr.
 *
 *	Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * ------------------------------------------------------------------------
 */
	STATUS
PartyTableGetFirst( AsnOid *oid, struct VarBindChoice *vbc, struct MibRecord *mr)
{
	struct PartyEntry	*pp;
	int		oidi;
	PField		field;
	STATUS		status;

	pp = PartyMIB.parties;
	field = PIDENTITY;
	if (!pp)
		if (mr->next)
			return((mr->next->func)(oid, vbc, mr->next, P_GET_FIRST));
		else {
			vbc->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
			free( vbc->a.value);
			return(FAILED);	/* end of mibview */
		}

	/* At least one instance of a table-entry exists for this table */
	if ((oid->octs = (unsigned char *) realloc(oid->octs,
(mr->OID.octetLen+1)+pp->partyIdentity->octetLen)) == NULL)
		return(FAILED);	
	oidi = mr->OID.octetLen;
	memcpy( (char *)oid->octs, (char *)mr->OID.octs, mr->OID.octetLen);
	*(oid->octs + oidi++) = (unsigned char) field;		/* partyIdentity(1) */
	if (pp->partyIdentity->octs[0] >= 40)
		*(oid->octs + oidi++) = pp->partyIdentity->octs[0]/40;
	memcpy( (oid->octs + oidi), pp->partyIdentity->octs, pp->partyIdentity->octetLen);
	*(oid->octs + oidi) = *(oid->octs + oidi)%40;
	oid->octetLen = oidi + pp->partyIdentity->octetLen;
	while(( status = (PartyTableFetch( field, pp, vbc))) == FAILED) {
		vbc->choiceId = VARBINDCHOICE_VALUE;
		/* if fetch fails this will fail for all instances of this field, try next field */
		if (++field <= PSTATUS)
			*(oid->octs + PARTYPREFIX) = (char) field;
		else	if (mr->next)
				return((mr->next->func)(oid, vbc, mr->next, P_GET_FIRST));
			else {
				vbc->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
				free( vbc->a.value);
				return(ERROR);	/* end of mibview */
			}
	}
	return(status);
}

/*
 * ------------------------------------------------------------------------
 *
 *	Function:	PartyTableGetNext
 *
 *	Arguments:	AsnOid			*oid	(IN)
 *			struct MibRecord	*mr	(IN)
 *			struct VarBindChoice	*vbc	(OUT)
 *
 *	Purpose:	Executes a GETNEXT operation on the managed object specified
 *			by oid in the partyTable and stores the result in the variable binding
 *			vbc. If the table is empty, routines are called for the 
 *			lexicographically next object using mr.
 *
 *	Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * ------------------------------------------------------------------------
 */
	STATUS
PartyTableGetNext( AsnOid *oid, VarBindChoice *vbc, struct MibRecord *mr)
{
	struct PartyEntry	*pp;
	AsnOid			*oidp;
	PField			field;
	int			oidi;
	STATUS			status;

	if (PartyMatch(oid, &field, &pp) != OK) {
		oidi = PARTYPREFIX;
		if (oidi < oid->octetLen)
			field = (PField) *((char *)(oid->octs + oidi++));
		else	field = PIDENTITY;

		oidp = pNewOid(oid->octetLen - oidi);
		if (( (oidi+1) < oid->octetLen) && ((oid->octs[oidi] <= 2) && ( oid->octs[oidi+1] <
40))  ){
			oidp->octetLen = (oid->octetLen - oidi) - 1;
			oidp->octs[0] = (oid->octs[oidi]*40) + oid->octs[oidi+1];
			memcpy( &oidp->octs[1], (oid->octs+(oidi+2)), oidp->octetLen-1);
		} else {
			oidp->octetLen = oid->octetLen - oidi;
			memcpy( oidp->octs, (oid->octs + oidi), oidp->octetLen);
		}
		if (field < PIDENTITY) {
			field = PIDENTITY;
			oidp->octetLen = 0;	/* as field increases, oidp becomes irrelevant */
		}
		while (field <= PSTATUS) {
			pp = PartyMIB.parties;
			while (pp) {
				if (CMPOID( oidp, pp->partyIdentity) == LESS)
					break;
				pp = pp->next;
			}
			if (pp)	break;
			field++;
			oidp->octetLen = 0;	/* as field increases, oidp becomes irrelevant */
		}
		FreeOid(oidp);
		if (field > PSTATUS)		/* no table-entry found */
			if (mr->next)
				return(mr->next->func)(oid, vbc, mr->next, P_GET_FIRST);
			else {
				vbc->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
				free( vbc->a.value);
				return(FAILED);	/* end of mibview */
			}
	} else {
		pp = pp->next;
		while (!pp)		/* no more instances of this table-object found */
			if (++field > PSTATUS)
				if (mr->next)
					return(mr->next->func)(oid, vbc, mr->next, P_GET_FIRST);
				else {
					vbc->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
					free( vbc->a.value);
					return(FAILED);	/* end of mibview */
				}
			else pp = PartyMIB.parties;	/* start with first instance again */
	}

	while (1) {
		if ((oid->octs = (unsigned char *)realloc( oid->octs,
(PARTYPREFIX+1)+pp->partyIdentity->octetLen)) == NULL)
			return(FAILED);
		oidi = PARTYPREFIX;
		*(oid->octs+oidi++) = (unsigned char) field;
		if (pp->partyIdentity->octs[0] >= 40)
			*(oid->octs + oidi++) = (pp->partyIdentity->octs[0]/40);
		memcpy( (oid->octs + oidi), pp->partyIdentity->octs, pp->partyIdentity->octetLen);
		*(oid->octs + oidi) = (pp->partyIdentity->octs[0]%40);
		oid->octetLen = oidi + pp->partyIdentity->octetLen;
		if ((status = (PartyTableFetch( field, pp, vbc))) != FAILED)
			return(status);
		vbc->choiceId = VARBINDCHOICE_VALUE;
		if ( ++field > PSTATUS)
	/* if fetch fails other partyIDs will fail too */
			if (mr->next)
				return(mr->next->func)(oid, vbc, mr->next, P_GET_FIRST);
			else {
				vbc->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
				free( vbc->a.value);
				return( FAILED);
			}
		pp = PartyMIB.parties;	/* start with first instance again */
	}
}

/*
 * -----------------------------------------------------------------------
 * Function:    PartyTableSet
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		struct VarBindChoice	*vbc	(IN)
 *
 * Purpose:     Executes a SET operation on the managed object specified by
 *		oid and uses the value in the variable binding vbc
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS
PartyTableSet( AsnOid *oid, VarBindChoice *vbc)
{
	PField			field;
	struct PartyEntry	*pp;

	if (PartyMatch( oid, &field, &pp) != OK)
		return(ERROR);
	else	return(PartyTableStore( field, pp, vbc));
}


/* 
 * -----------------------------------------------------------------------
 * Function:    PartyTableStore
 *
 * Arguments:   PField			field	(IN)
 *		struct PartyEntry	*pp	(IN)
 *		struct VarBindChoice	*vbc	(IN)
 *
 * Purpose:     Stores the value specified in the variable binding vbc in the
 *		managed object specified by its field and entry in the partyTable pp.
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS
PartyTableStore( PField field, struct PartyEntry *pp, struct VarBindChoice *vbc)
{
	ObjectSyntax	*object;

	object = vbc->a.value;
	switch(field) {
	case PIDENTITY: /* not-accessible */
		errno = NOTWRITABLE;
		return(ERROR);

	case PINDEX: /* read-only */
		errno = NOTWRITABLE;
		return(ERROR);

	case PTDOMAIN: /* read-create */
		if (object->choiceId == OBJECTSYNTAX_OBJECTID_VALUE) {
			pp->partyTDomain->octetLen = object->a.objectID_value->octetLen;
			if ((pp->partyTDomain->octs = (unsigned char *)realloc(pp->partyTDomain->octs,
pp->partyTDomain->octetLen)) == NULL) {
				errno = RESOURCEUNAVAILABLE;
				return(ERROR);
			}
			memcpy( pp->partyTDomain->octs, object->a.objectID_value->octs,
pp->partyTDomain->octetLen);
			return(OK);
		} else {
			errno = WRONGVALUE;
			return(ERROR);
		}

	case PTADDRESS: /* read-create */
		if (object->choiceId == OBJECTSYNTAX_STRING_VALUE) {
			pp->partyTAddress->octetLen = object->a.string_value->octetLen;
			if ((pp->partyTAddress->octs = (unsigned char*)realloc(pp->partyTAddress->octs,
pp->partyTAddress->octetLen)) == NULL) {
				errno = RESOURCEUNAVAILABLE;
				return(ERROR);
			}
			memcpy( pp->partyTAddress->octs, object->a.string_value->octs,
pp->partyTAddress->octetLen);
			return(OK);
		} else {
			errno = WRONGVALUE;
			return(ERROR);
		}

	case PMAXMESSAGESIZE: /* read-create */
		if (object->choiceId == OBJECTSYNTAX_INTEGER_VALUE) {
			pp->partyMaxMessageSize = object->a.integer_value;
			return(OK);
		} else {
			errno = WRONGVALUE;
			return(ERROR);
		}

	case PLOCAL: /* read-create */
		if (object->choiceId == OBJECTSYNTAX_INTEGER_VALUE) {
			pp->partyLocal = object->a.integer_value;
			return(OK);
		} else {
			errno = WRONGVALUE;
			return(ERROR);
		}

	case PAUTHPROTOCOL: /* once an instance of this object is created, its value can not be
changed */
		errno = NOTWRITABLE;
		return(ERROR);
		
	case PAUTHCLOCK: /* read-create; no decrementation unless priv auth key is changed
simultaneously */
		if (object->choiceId == OBJECTSYNTAX_UNSIGNED_INTEGER_VALUE) {
			pp->partyAuthClock = object->a.unsigned_integer_value;
			return(OK);
		} else {
			errno = WRONGVALUE;
			return(ERROR);
		}

	case PAUTHPRIVATE: /* read-create; XOR-ed value */
		if (object->choiceId == OBJECTSYNTAX_STRING_VALUE) {
			if (!pp->partyAuthPrivate)
				if ((pp->partyAuthPrivate = pNewOct(KEYLEN)) == NULL) {
					errno = RESOURCEUNAVAILABLE;
					return(ERROR);
				}
			pp->partyAuthPrivate->octetLen = object->a.string_value->octetLen;
			if ((pp->partyAuthPrivate->octs = (unsigned char *)realloc(
pp->partyAuthPrivate->octs, pp->partyAuthPrivate->octetLen)) == NULL) {
				errno = RESOURCEUNAVAILABLE;
				return(ERROR);
			}
			memcpy( pp->partyAuthPrivate->octs, object->a.string_value->octs,
pp->partyAuthPrivate->octetLen);
			return(OK);
		} else {
			errno = WRONGVALUE;
			return(ERROR);
		}

	case PAUTHPUBLIC: /* read-create */
		if (object->choiceId == OBJECTSYNTAX_STRING_VALUE) {
			if (!pp->partyAuthPublic)
				if ((pp->partyAuthPublic = pNewOct(KEYLEN)) == NULL) {
					errno = RESOURCEUNAVAILABLE;
					return(ERROR);
				}
			pp->partyAuthPublic->octetLen = object->a.string_value->octetLen;
			if ((pp->partyAuthPublic->octs = (unsigned char *)realloc(
pp->partyAuthPublic->octs, pp->partyAuthPublic->octetLen)) == NULL) {
				errno = RESOURCEUNAVAILABLE;
				return(ERROR);
			}
			memcpy( pp->partyAuthPublic->octs, object->a.string_value->octs,
pp->partyAuthPublic->octetLen);
			return(OK);
		} else {
			errno = WRONGVALUE;
			return(ERROR);
		}

	case PAUTHLIFETIME: /* read-create; once an instance of this object is created, its value can not be
changed */
		errno = NOTWRITABLE;
		return(ERROR);

	case PPRIVPROTOCOL: /* read-create; once an instance of this object is created, its value can not be
changed */
		errno = NOTWRITABLE;
		return(ERROR);

	case PPRIVPRIVATE: /* read-create; XOR-ed value */
		if (object->choiceId == OBJECTSYNTAX_STRING_VALUE) {
			pp->partyPrivPrivate->octetLen = object->a.string_value->octetLen;
			if ((pp->partyPrivPrivate->octs = (unsigned char *)realloc(
pp->partyPrivPrivate->octs, pp->partyPrivPrivate->octetLen)) == NULL) {
				errno = RESOURCEUNAVAILABLE;
				return(ERROR);
			}
			memcpy( pp->partyPrivPrivate->octs, object->a.string_value->octs,
pp->partyPrivPrivate->octetLen);
			return(OK);
		} else {
			errno = WRONGVALUE;
			return(ERROR);
		}

	case PPRIVPUBLIC: /* read-create */
		if (object->choiceId == OBJECTSYNTAX_STRING_VALUE) {
			pp->partyPrivPublic->octetLen = object->a.string_value->octetLen;
			if ((pp->partyPrivPublic->octs = (unsigned char *)realloc(
pp->partyPrivPublic->octs, pp->partyPrivPublic->octetLen)) == NULL) {
				errno = RESOURCEUNAVAILABLE;
				return(ERROR);
			}
			memcpy( pp->partyPrivPublic->octs, object->a.string_value->octs,
pp->partyPrivPublic->octetLen);
			return(OK);
		} else {
			errno = WRONGVALUE;
			return(ERROR);
		}

	case PCLONEFROM: /* read-create; once an instance of this object is created, its value can not be
changed */
		errno = NOTWRITABLE;
		return(ERROR);

	case PSTORAGETYPE: /* read-create */
		if (object->choiceId == OBJECTSYNTAX_INTEGER_VALUE) {
			pp->partyStorageType = object->a.integer_value;
			return(OK);
		} else {
			errno = WRONGVALUE;
			return(ERROR);
		}

	case PSTATUS: /* read-create */
		if (object->choiceId == OBJECTSYNTAX_INTEGER_VALUE) {
			pp->partyStatus = object->a.integer_value;
			return(OK);
		} else {
			errno = WRONGVALUE;
			return(ERROR);
		}
		
	default:
		errno = NOCREATION;
		break;
	}
	return(ERROR);

}

/*
 * -----------------------------------------------------------------------
 * Function:    ContextTableGet
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     Executes a GET operation on the managed object specified by
 *		oid and stores the result in the variable binding vbc
 *
 * Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * -----------------------------------------------------------------------
 */
	STATUS
ContextTableGet( AsnOid *oid, VarBindChoice *vbc)
{
	CField			field;
	struct ContextEntry	*cp;

	if (ContextMatch( oid, &field, &cp) != OK) {
		free( vbc->a.value);
		return(FAILED);
	} else	return(ContextTableFetch( field, cp, vbc));
}


/*
 * -----------------------------------------------------------------------
 * Function:    ContextMatch
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		CField			*field	(OUT)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     This routine searches the contextTable for an entry which
 *		matches oid. The found entry and field are returned
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS
ContextMatch( AsnOid *oid, CField *field, struct ContextEntry **cp)
{
	int		oidi, loc;
	Context		*oidp;
	struct Hashnode	*phn;

	oidi = PARTYPREFIX;

	if ((*field = *(char *)(oid->octs + oidi++)) > CSTATUS)
		return(ERROR);
        oidp = pNewOid(oid->octetLen - (PARTYPREFIX + 1));
        if ((oid->octs[PARTYPREFIX +1] <= 2) && ( oid->octs[PARTYPREFIX+2] < 40)) {
                oidp->octetLen = oid->octetLen - (PARTYPREFIX + 2);
                oidp->octs[0] = (oid->octs[PARTYPREFIX+1]*40) + oid->octs[PARTYPREFIX+2];
                memcpy( &oidp->octs[1], (oid->octs+(PARTYPREFIX + 3)), oidp->octetLen-1);
        } else {
                oidp->octetLen = oid->octetLen - (PARTYPREFIX + 1);
                memcpy( oidp->octs, (oid->octs + PARTYPREFIX + 2), oid->octetLen);
        }

	loc = hashoid( oidp, csize, cradix);
	phn = chashtab[ loc];
	while (phn != NULL) {
		if (CMPOID( oidp, ((ContextEntry *)phn->pentry)->contextIdentity) == EQUAL) {
			*cp = phn->pentry;
			FreeOid(oidp);
			return(OK);
		} else	phn = phn->pnext;
	}
	FreeOid(oidp);
	return(FAILED);
}



/*
 * -----------------------------------------------------------------------
 * Function:    ContextTableFetch
 *
 * Arguments:   CField			field	(IN)
 *		struct ContextEntry	*cp	(IN)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     Fetches the columnar object instance specified by field from
 *		the contextEntry specified by cp and stores the result in vbc
 *
 * Remarks:	Allocation of memory for values with variable size is done
 *		inside this routine
 *
 * -----------------------------------------------------------------------
 */
	STATUS
ContextTableFetch( CField field, ContextEntry *cp, VarBindChoice *vbc)
{
	ObjectSyntax	*object;

	object = vbc->a.value;
	switch(field) {

	case CIDENTITY: /* not accessible */
		vbc->choiceId = VARBINDCHOICE_NOSUCHINSTANCE;
		return(FAILED);

	case CINDEX: /* read-only */
		object->choiceId = OBJECTSYNTAX_INTEGER_VALUE;
		object->a.integer_value = cp->contextIndex;
		return(OK);

	case CLOCAL: /* read-create */
		object->choiceId = OBJECTSYNTAX_INTEGER_VALUE;
		if (cp->contextLocal)
			object->a.integer_value = cp->contextLocal;
		else {
			return(FAILED);
		}
		return(OK);
	
	case CVIEWINDEX: /* read-create */
		object->choiceId = OBJECTSYNTAX_INTEGER_VALUE;
		object->a.integer_value = cp->contextViewIndex;
		return(OK);
		
	case CLOCALENTITY: /* read-create */
		object->choiceId = OBJECTSYNTAX_STRING_VALUE;
		if (cp->contextLocalEntity) {
			if ((object->a.string_value = pNewOct( cp->contextLocalEntity->octetLen)) ==
NULL)
				return(ERROR);
			object->a.string_value->octetLen = cp->contextLocalEntity->octetLen;
			memcpy( object->a.string_value->octs, cp->contextLocalEntity->octs,
object->a.string_value->octetLen);
		} else {
			vbc->choiceId = VARBINDCHOICE_NOSUCHINSTANCE;
			return(FAILED);
		}
		return(OK);
		
	case CLOCALTIME: /* read-create */
		object->choiceId = OBJECTSYNTAX_OBJECTID_VALUE;
		if (cp->contextLocalTime) {
			object->a.objectID_value = pNewOid( cp->contextLocalTime->octetLen);
			object->a.objectID_value->octetLen = cp->contextLocalTime->octetLen;
			memcpy( object->a.objectID_value->octs, cp->contextLocalTime->octs,
object->a.objectID_value->octetLen);			
		} else {
			vbc->choiceId = VARBINDCHOICE_NOSUCHINSTANCE;
			return(FAILED);
		}
		return(OK);

	case CPROXYDSTPARTY: /* read-create */
		object->choiceId = OBJECTSYNTAX_OBJECTID_VALUE;
		if (cp->contextProxyDstParty) {
			object->a.objectID_value = pNewOid( cp->contextProxyDstParty->octetLen);
			object->a.objectID_value->octetLen = cp->contextProxyDstParty->octetLen;
			memcpy( object->a.objectID_value->octs, cp->contextProxyDstParty->octs,
object->a.objectID_value->octetLen); 
		} else {
			vbc->choiceId = VARBINDCHOICE_NOSUCHINSTANCE;
			return(FAILED);
		}
		return(OK);

	case CPROXYSRCPARTY: /* read-create */
		object->choiceId = OBJECTSYNTAX_OBJECTID_VALUE;
		if (cp->contextProxySrcParty) {
			object->a.objectID_value = pNewOid( cp->contextProxySrcParty->octetLen);
			object->a.objectID_value->octetLen = cp->contextProxySrcParty->octetLen;
			memcpy( object->a.objectID_value->octs, cp->contextProxySrcParty->octs,
object->a.objectID_value->octetLen);
		} else {
			vbc->choiceId = VARBINDCHOICE_NOSUCHINSTANCE;
			return(FAILED);
		}
		return(OK);

	case CPROXYCONTEXT: /* read-create */
		object->choiceId = OBJECTSYNTAX_OBJECTID_VALUE;
		if (cp->contextProxyContext) {
			object->a.objectID_value = pNewOid(cp->contextProxyContext->octetLen);
			object->a.objectID_value->octetLen = cp->contextProxyContext->octetLen;
			memcpy( object->a.objectID_value->octs, cp->contextProxyContext->octs,
object->a.objectID_value->octetLen);
		} else {
			vbc->choiceId = VARBINDCHOICE_NOSUCHINSTANCE;
			return(FAILED);
		}
		return(OK);

	case CSTORAGETYPE:
		object->choiceId = OBJECTSYNTAX_INTEGER_VALUE;
		if (cp->contextStorageType)
			object->a.integer_value = cp->contextStorageType;
		else {
			vbc->choiceId = VARBINDCHOICE_NOSUCHINSTANCE;
			return(FAILED);
		}
		return(OK);

	case CSTATUS: /* read-create */
		object->choiceId = OBJECTSYNTAX_INTEGER_VALUE;
		if (cp->contextStatus)
			object->a.integer_value = cp->contextStatus;
		else {
			vbc->choiceId = VARBINDCHOICE_NOSUCHINSTANCE;
			return(FAILED);
		}
		return(OK);
		
	default:
		vbc->choiceId = NOSUCHOBJECT;
		break;
	}
	return(ERROR);
}


/*
 * ------------------------------------------------------------------------
 *
 *	Function:	ContextTableGetFirst
 *
 *	Arguments:	AsnOid			*oid	(IN)
 *			struct MibRecord	*mr	(IN)
 *			struct VarBindChoice	*vbc	(OUT)
 *
 *	Purpose:	Finds the lexicographically first managed object in the
 *			contextTable and stores the result in the variable binding
 *			vbc. If the table is empty, routines are called for the 
 *			lexicographically next object using mr.
 *
 *	Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * ------------------------------------------------------------------------
 */
	STATUS
ContextTableGetFirst( AsnOid *oid, VarBindChoice *vbc, struct MibRecord *mr)
{
	struct ContextEntry	*cp;
	int			i;
	CField			field;
	STATUS			status;

	cp = PartyMIB.contexts;
	field = (CField) CIDENTITY;
	if (!cp)
		if (mr->next)
			return((mr->next->func)(oid, vbc, mr->next, P_GET_FIRST));
		else {
			vbc->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
			free( vbc->a.value);
			return(FAILED);	/* end of mibview */
		}

	/* At least one instance of a table-entry exists for this table */
	if ((oid->octs = (unsigned char *)realloc(oid->octs,
(mr->OID.octetLen+1)+cp->contextIdentity->octetLen)) == NULL)
		return(ERROR);
	i = mr->OID.octetLen;
	memcpy( (char *)oid->octs, (char *)mr->OID.octs, i);
	*(oid->octs + i++) = (char) field;
	if (cp->contextIdentity->octs[0] >= 40)
		*(oid->octs + i++) = (cp->contextIdentity->octs[0]/40);
	memcpy( oid->octs+i, cp->contextIdentity->octs, cp->contextIdentity->octetLen);
	*(oid->octs + i) = *(oid->octs + i)%40;
	oid->octetLen = i + cp->contextIdentity->octetLen;
	while ((status = (ContextTableFetch( field, cp, vbc))) != OK) {
		vbc->choiceId = VARBINDCHOICE_VALUE;
		/* if fetch fails this will fail for all instances of this field, try next field */
		if (++field <= CSTATUS)
			*(oid->octs + PARTYPREFIX) = (char) field;
		else	if (mr->next) {
				return((mr->next->func)(oid, vbc, mr->next, P_GET_FIRST));
			} else {
				vbc->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
				free( vbc->a.value);
				return(FAILED);
			}
	}
	return(status);
}

/*
 * ------------------------------------------------------------------------
 *
 *	Function:	ContextTableGetNext
 *
 *	Arguments:	AsnOid			*oid	(IN)
 *			struct MibRecord	*mr	(IN)
 *			struct VarBindChoice	*vbc	(OUT)
 *
 *	Purpose:	Executes a GETNEXT operation on the managed object specified
 *			by oid in the contextTable and stores the result in the variable binding
 *			vbc. If the table is empty, routines are called for the 
 *			lexicographically next object using mr.
 *
 *	Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * ------------------------------------------------------------------------
 */
	STATUS
ContextTableGetNext(AsnOid *oid, VarBindChoice *vbc, struct MibRecord *mr)
{
	struct ContextEntry	*cp;
	AsnOid			*oidp;
	CField			field;
	STATUS			status;
	int			oidi;

	if (ContextMatch( oid, &field, &cp) != OK) {
		oidi = PARTYPREFIX;
		if (oidi < oid->octetLen)
			field = (CField)*((char *)(oid->octs + oidi++));
		else	field = CIDENTITY;
                oidp = pNewOid(oid->octetLen - oidi);
                if (( ( oidi+1) < oid->octetLen) && ((oid->octs[oidi] <= 2) && ( oid->octs[oidi+1]
< 40)) ) {
                        oidp->octetLen = (oid->octetLen - oidi) - 1;
                        oidp->octs[0] = (oid->octs[oidi]*40) + oid->octs[oidi+1];
                        memcpy( &oidp->octs[1], (oid->octs+(oidi+2)), oidp->octetLen-1);
                } else {
                        oidp->octetLen = oid->octetLen - oidi;
                        memcpy( oidp->octs, (oid->octs + oidi), oidp->octetLen);
                }
		if (field < CIDENTITY) {
			field = CIDENTITY;
			oidp->octetLen = 0;
		}
		while (field <= CSTATUS) {
			cp = PartyMIB.contexts;
			while (cp) {
				if (CMPOID( oidp, cp->contextIdentity) == LESS)
					break;
				cp = cp->next;
			}
			if (cp)	break;
			field++;
			oidp->octetLen = 0;
		}
		FreeOid(oidp);
		if (field > CSTATUS)
			if (mr->next)
				return(mr->next->func)(oid, vbc, mr->next, P_GET_FIRST);
			else {
				vbc->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
				free( vbc->a.value);
				return(FAILED);
			}
	} else {
		cp = cp->next;
		while(!cp)
			if (++field > CSTATUS)
				if (mr->next)
					return(mr->next->func)(oid, vbc, mr->next, P_GET_FIRST);
				else {
					vbc->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
					free( vbc->a.value);
					return(FAILED);
				}
			else	cp = PartyMIB.contexts;
	}

	while (1) {
		oidi = PARTYPREFIX;
		if ((oid->octs = (unsigned char *)realloc(oid->octs,
(PARTYPREFIX+1)+cp->contextIdentity->octetLen)) == NULL)
			return(ERROR);
		*(oid->octs + oidi++) = (unsigned char) field;
        	if (cp->contextIdentity->octs[0] >= 40)
               		*(oid->octs + oidi++) = (cp->contextIdentity->octs[0]/40);
                memcpy( (oid->octs + oidi), cp->contextIdentity->octs,
cp->contextIdentity->octetLen);
                *(oid->octs + oidi) = (cp->contextIdentity->octs[0]%40);
                oid->octetLen = oidi + cp->contextIdentity->octetLen;
		if ((status = ( ContextTableFetch( field, cp, vbc))) != FAILED)
			return(status);
		vbc->choiceId = VARBINDCHOICE_VALUE;
		if (++field > CSTATUS)
			if (mr->next)
				return(mr->next->func)(oid, vbc, mr->next, P_GET_FIRST);
			else {
				vbc->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
				free( vbc->a.value);
				return(FAILED);
			}
		cp = PartyMIB.contexts;
	}
}

/*
 * -----------------------------------------------------------------------
 * Function:    ContextTableSet
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		struct VarBindChoice	*vbc	(IN)
 *
 * Purpose:     Executes a SET operation on the managed object specified by
 *		oid and uses the value in the variable binding vbc
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS
ContextTableSet( AsnOid *oid, VarBindChoice *vbc)
{
	CField			field;
	struct ContextEntry	*cp;

	if (ContextMatch( oid, &field, &cp) != OK)
		return(ERROR);
	else	return(ContextTableStore( field, cp, vbc));
}


/* 
 * -----------------------------------------------------------------------
 * Function:    ContextTableStore
 *
 * Arguments:   CField			field	(IN)
 *		struct ContextEntry	*cp	(IN)
 *		struct VarBindChoice	*vbc	(IN)
 *
 * Purpose:     Stores the value specified in the variable binding vbc in the
 *		managed object specified by its field and entry in the contextTable cp.
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS
ContextTableStore( CField field, ContextEntry *cp, VarBindChoice *vbc)
{
	ObjectSyntax	*object;

	object = vbc->a.value;
	switch(field) {

	case CIDENTITY: /* not accessible */
                errno = NOTWRITABLE;
                return(ERROR);

	case CINDEX: /* read-only */
                errno = NOTWRITABLE;
                return(ERROR);

	case CLOCAL: /* read-create */
                if (object->choiceId == OBJECTSYNTAX_INTEGER_VALUE) {
                        cp->contextLocal = object->a.integer_value;
                        return(OK);
                } else {
                        errno = WRONGVALUE;
                        return(ERROR);
                }
	
	case CVIEWINDEX: /* read-create */
		if (object->choiceId == OBJECTSYNTAX_INTEGER_VALUE) {
			cp->contextViewIndex = object->a.integer_value;
			return(OK);
                } else {
                        errno = WRONGVALUE;
                        return(ERROR);
                }
		
	case CLOCALENTITY: /* read-create */
                if (object->choiceId == OBJECTSYNTAX_STRING_VALUE) {
                        cp->contextLocalEntity->octetLen = object->a.string_value->octetLen;
                        if ((cp->contextLocalEntity->octs = (unsigned char
*)realloc(cp->contextLocalEntity->octs, cp->contextLocalEntity->octetLen)) == NULL) {
				errno = RESOURCEUNAVAILABLE;
				return(ERROR);
			}
                        memcpy( cp->contextLocalEntity->octs, object->a.string_value->octs,
cp->contextLocalEntity->octetLen);
                        return(OK);
                } else {
                        errno = WRONGVALUE;
                        return(ERROR);
                }
		
	case CLOCALTIME: /* read-create */
                if (object->choiceId == OBJECTSYNTAX_OBJECTID_VALUE) {
                        cp->contextLocalTime->octetLen = object->a.objectID_value->octetLen;
                        if ((cp->contextLocalTime->octs = (unsigned char
*)realloc(cp->contextLocalTime->octs, cp->contextLocalTime->octetLen)) == NULL) {
				errno = RESOURCEUNAVAILABLE;
				return(ERROR);
			}
                        memcpy( cp->contextLocalTime->octs, object->a.objectID_value->octs,
cp->contextLocalTime->octetLen);
                        return(OK);
                } else {
                        errno = WRONGVALUE;
                        return(ERROR);
                }

	case CPROXYDSTPARTY: /* read-create */
                if (object->choiceId == OBJECTSYNTAX_OBJECTID_VALUE) {
                        cp->contextProxyDstParty->octetLen = object->a.objectID_value->octetLen;
                        if ((cp->contextProxyDstParty->octs = (unsigned char *)realloc(cp->contextProxyDstParty->octs,
cp->contextProxyDstParty->octetLen)) == NULL) {
				errno = RESOURCEUNAVAILABLE;
				return(ERROR);
			}
                        memcpy( cp->contextProxyDstParty->octs, object->a.objectID_value->octs,
cp->contextProxyDstParty->octetLen);
                        return(OK);
                } else {
                        errno = WRONGVALUE;
                        return(ERROR);
                }

	case CPROXYSRCPARTY: /* read-create */
                if (object->choiceId == OBJECTSYNTAX_OBJECTID_VALUE) {
                        cp->contextProxySrcParty->octetLen = object->a.objectID_value->octetLen;
                        if ((cp->contextProxySrcParty->octs = (unsigned char *)realloc(cp->contextProxySrcParty->octs,
cp->contextProxySrcParty->octetLen)) == NULL) {
				errno = RESOURCEUNAVAILABLE;
				return(ERROR);
			}
                        memcpy( cp->contextProxySrcParty->octs, object->a.objectID_value->octs,
cp->contextProxySrcParty->octetLen);
                        return(OK);
                } else {
                        errno = WRONGVALUE;
                        return(ERROR);
                }

	case CPROXYCONTEXT: /* read-create */
                if (object->choiceId == OBJECTSYNTAX_OBJECTID_VALUE) {
                        cp->contextProxyContext->octetLen = object->a.objectID_value->octetLen;
                        if ((cp->contextProxyContext->octs = (unsigned char
*)realloc(cp->contextProxyContext->octs, cp->contextProxyContext->octetLen)) == NULL) {
				errno = RESOURCEUNAVAILABLE;
				return(ERROR);
			}
                        memcpy( cp->contextProxyContext->octs, object->a.objectID_value->octs,
cp->contextProxyContext->octetLen);
                        return(OK);
                } else {
                        errno = WRONGVALUE;
                        return(ERROR);
                }

	case CSTORAGETYPE:
                if (object->choiceId == OBJECTSYNTAX_INTEGER_VALUE) {
                        cp->contextStorageType = object->a.integer_value;
                        return(OK);
                } else {
                        errno = WRONGVALUE;
                        return(ERROR);
                }

	case CSTATUS: /* read-create */
                if (object->choiceId == OBJECTSYNTAX_INTEGER_VALUE) {
                        cp->contextStatus = object->a.integer_value;
                        return(OK);
                } else {
                        errno = WRONGVALUE;
                        return(ERROR);
                }
		
	default:
		errno = NOCREATION;
		break;
	}
	return(ERROR);
}


/*
 * -----------------------------------------------------------------------
 * Function:    AclTableGet
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     Executes a GET operation on the managed object specified by
 *		oid and stores the result in the variable binding vbc
 *
 * Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AclTableGet( AsnOid *oid, VarBindChoice *vbc)
{
	AField		field;
	struct AclEntry	*ap;

	if (AclMatch( oid, &field, &ap) != OK)
		return(OK);
	else	return( AclTableFetch( field, ap, vbc->a.value));
}


/*
 * -----------------------------------------------------------------------
 * Function:    AclMatch
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		AField			*field	(OUT)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     This routine searches the aclTable for an entry which
 *		matches oid. The found entry and field are returned
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AclMatch( AsnOid *oid, AField *field, AclEntry **ap)
{
	struct AclEntry	*pap;
	int		oidi;
	short		target, subject, resources;

	oidi = PARTYPREFIX;

	if ((*field = *(char *)(oid->octs + oidi++)) > ASTATUS)
		return(ERROR);
	target = *(char *)(oid->octs + oidi++);
	subject = *(char *)(oid->octs + oidi++);
	resources = *(char *)(oid->octs + oidi++);
	pap = PartyMIB.acls;
	while (pap) {
		if ( ((PartyEntry *)pap->aclTarget)->partyIndex > target)
			break;
		if ( ((PartyEntry *)pap->aclTarget)->partyIndex == target) {
			if ( ((PartyEntry *)pap->aclSubject)->partyIndex > subject)
				break;
			if ( ((PartyEntry *)pap->aclSubject)->partyIndex == subject) {
				if (((ContextEntry *)pap->aclResources)->contextIndex > resources)
					break;
				if ( ((ContextEntry *)pap->aclResources)->contextIndex == resources) {
					*ap = pap;
					return(OK);
				}
			}
		}
		pap = pap->next;
	}
	return(FAILED);
}

/*
 * -----------------------------------------------------------------------
 * Function:    AclTableFetch
 *
 * Arguments:   AField			field	(IN)
 *		struct AclEntry		*ap	(IN)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     Fetches the columnar object instance specified by field from
 *		the aclEntry specified by ap and stores the result in vbc
 *
 * Remarks:	Allocation of memory for values with variable size is done
 *		inside this routine
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AclTableFetch( AField field, AclEntry *ap, ObjectSyntax *object)
{
	switch( field) {

	case ATARGET: /* not-accessible */
		return(FAILED);

	case ASUBJECT: /* not-accessible */
		return(FAILED);

	case ARESOURCES: /* not-accessible */
		return(FAILED);

	case APRIVILEGES: /* read-create */
		object->choiceId = OBJECTSYNTAX_INTEGER_VALUE;
		if (ap->aclPrivileges)
			object->a.integer_value = ap->aclPrivileges;
		else	object->a.integer_value = 35;
		return(OK);

	case ASTORAGETYPE: /* read-create */
		object->choiceId = OBJECTSYNTAX_INTEGER_VALUE;
		if (ap->aclStorageType)
			object->a.integer_value = ap->aclStorageType;
		else	object->a.integer_value = nonVolatile;
		return(OK);

	case ASTATUS: /* read-create */
		object->choiceId = OBJECTSYNTAX_INTEGER_VALUE;
		if (ap->aclStatus)
			object->a.integer_value = ap->aclStatus;
		else	object->a.integer_value = notReady;
		return(OK);
		
	default:
		break;
	}
	return(ERROR);
}


/*
 * ------------------------------------------------------------------------
 *
 *	Function:	AclTableGetFirst
 *
 *	Arguments:	AsnOid			*oid	(IN)
 *			struct MibRecord	*mr	(IN)
 *			struct VarBindChoice	*vbc	(OUT)
 *
 *	Purpose:	Finds the lexicographically first managed object in the
 *			aclTable and stores the result in the variable binding
 *			vbc. If the table is empty, routines are called for the 
 *			lexicographically next object using mr.
 *
 *	Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * ------------------------------------------------------------------------
 */
	STATUS
AclTableGetFirst(AsnOid *oid, VarBindChoice *vbc, struct MibRecord *mr)
{
	struct AclEntry	*ap;
	int		oidi;
	AField		field;
	STATUS		status;

	ap = PartyMIB.acls;
	field = ATARGET;
	if (!ap)
		if (mr->next)
			return((mr->next->func)(oid, vbc, mr->next, P_GET_FIRST));
		else {
                        vbc->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
                        return(FAILED); /* end of mibview */
                }

        /* At least one instance of a table-entry exists for this table */
	if ((oid->octs = (unsigned char *)realloc( oid->octs, (mr->OID.octetLen+1)+3)) == NULL)
		return(ERROR);
	oidi = mr->OID.octetLen;
	memcpy( oid->octs, mr->OID.octs, mr->OID.octetLen);
	*(oid->octs + oidi++) = (char) field;
	*(oid->octs + oidi++) = (char) ((PartyEntry *)ap->aclTarget)->partyIndex;
	*(oid->octs + oidi++) = (char) ((PartyEntry *)ap->aclSubject)->partyIndex;
	*(oid->octs + oidi++) = (char) ((ContextEntry *)ap->aclResources)->contextIndex;
	oid->octetLen = oidi;
	while (( status = ( AclTableFetch( field, ap, vbc->a.value))) == FAILED)
		/* if fetch fails this will fail for all instances of this field, try next field */
		if (++field <= ASTATUS)
			*(oid->octs + PARTYPREFIX) = (char) field;
		else	if (mr->next)
				return((mr->next->func)(oid, vbc, mr->next, P_GET_FIRST));
			else {
                                vbc->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
                                return(FAILED);
                        }
	return(status);
}


/*
 * ------------------------------------------------------------------------
 *
 *	Function:	AclTableGetNext
 *
 *	Arguments:	AsnOid			*oid	(IN)
 *			struct MibRecord	*mr	(IN)
 *			struct VarBindChoice	*vbc	(OUT)
 *
 *	Purpose:	Executes a GETNEXT operation on the managed object specified
 *			by oid in the aclTable and stores the result in the variable binding
 *			vbc. If the table is empty, routines are called for the 
 *			lexicographically next object using mr.
 *
 *	Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * ------------------------------------------------------------------------
 */
	STATUS
AclTableGetNext( AsnOid *oid, VarBindChoice *vbc, struct MibRecord *mr)
{
	struct AclEntry		*ap;
	AField			field;
	short			target, subject, resources;
	STATUS			status;
	int			oidi;

	field = ATARGET;
	target = 0; subject = 0; resources = 0;
	if ( AclMatch( oid, &field, &ap) != OK) {
		oidi = PARTYPREFIX;
		if (oidi < oid->octetLen) {
			field = (AField)*((char *)(oid->octs + oidi++));
			if (oidi < oid->octetLen) {
				target = *(char *)(oid->octs + oidi++);
				if (oidi < oid->octetLen) {
					subject = *(char *)(oid->octs + oidi++);
					if (oidi < oid->octetLen)
						resources = *(char *)(oid->octs + oidi++);
				}
			}
		}
		while (field <= ASTATUS) {
			ap = PartyMIB.acls;
			while (ap) {
				if ( ((PartyEntry *)ap->aclTarget)->partyIndex > target)
					break;
				if ( ((PartyEntry *)ap->aclTarget)->partyIndex == target) {
					if ( ((PartyEntry *)ap->aclSubject)->partyIndex > subject)
						break;
					if ( ((PartyEntry *)ap->aclSubject)->partyIndex == subject)
						if ( ((ContextEntry *)ap->aclResources)->contextIndex >= resources)
							break;
				}
				ap = ap->next;	
			}
			if (ap)	break;
			field++;
			target = 0;
		}
		if (field > ASTATUS)
			if (mr->next)
				return((mr->next->func)(oid, vbc, mr->next, P_GET_FIRST));
			else {
                                vbc->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
                                return(FAILED);
                        }
	} else	{
		ap = ap->next;
		while (!ap)
			if (++field > ASTATUS)
				if (mr->next)
					return((mr->next->func)(oid, vbc, mr->next, P_GET_FIRST));
				else {
                                	vbc->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
                               		return(FAILED);
                     	   	}
			else	ap = PartyMIB.acls;
	}

	while (1) {
		if ((oid->octs = (unsigned char *)realloc( oid->octs, (PARTYPREFIX+1)+3)) == NULL)
			return(ERROR);
		oidi = PARTYPREFIX;
		*(oid->octs+oidi++) = (char) field;
		*(oid->octs+oidi++) = (char)((PartyEntry *)ap->aclTarget)->partyIndex;
		*(oid->octs+oidi++) = (char)((PartyEntry *)ap->aclSubject)->partyIndex;
		*(oid->octs+oidi++) = (char)((ContextEntry *)ap->aclResources)->contextIndex;
		if ((status = AclTableFetch( field, ap, vbc->a.value)) != FAILED)
			return(status);
		/* if fetch fails this will fail for all instances of this field, try next field */
		if (++field > ASTATUS)
			if (mr->next)
				return((mr->next->func)(oid, vbc, mr->next, P_GET_FIRST));
			else {
                                vbc->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
                                return( FAILED);
                        }
		ap = PartyMIB.acls;
	}
}

/*
 * -----------------------------------------------------------------------
 * Function:    AclTableSet
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		struct VarBindChoice	*vbc	(IN)
 *
 * Purpose:     Executes a SET operation on the managed object specified by
 *		oid and uses the value in the variable binding vbc
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AclTableSet( AsnOid *oid, VarBindChoice *vbc)
{
	AField		field;
	struct AclEntry	*ap;

	if (AclMatch( oid, &field, &ap) != OK)
		return(ERROR);
	else	return(AclTableStore( field, ap, vbc));
}

/* 
 * -----------------------------------------------------------------------
 * Function:    AclTableStore
 *
 * Arguments:   AField			field	(IN)
 *		struct AclEntry		*ap	(IN)
 *		struct VarBindChoice	*vbc	(IN)
 *
 * Purpose:     Stores the value specified in the variable binding vbc in the
 *		managed object specified by its field and entry in the aclEntry ap.
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AclTableStore( AField field, struct AclEntry *ap, VarBindChoice *vbc)
{
	ObjectSyntax	*object;

	object = vbc->a.value;
	switch( field) {

	case ATARGET: /* not-accessible */
                errno = NOTWRITABLE;
                return(ERROR);

	case ASUBJECT: /* not-accessible */
                errno = NOTWRITABLE;
                return(ERROR);

	case ARESOURCES: /* not-accessible */
                errno = NOTWRITABLE;
                return(ERROR);

	case APRIVILEGES: /* read-create */
        	if (object->choiceId == OBJECTSYNTAX_INTEGER_VALUE) {
            		ap->aclPrivileges = object->a.integer_value;
			return(OK);
        	} else {
                        errno = WRONGVALUE;
                        return(ERROR);
                }

	case ASTORAGETYPE: /* read-create */
        	if (object->choiceId == OBJECTSYNTAX_INTEGER_VALUE) {
            		ap->aclStorageType = object->a.integer_value;
			return(OK);
        	} else {
                        errno = WRONGVALUE;
                        return(ERROR);
                }

    	case ASTATUS: /* read-create */
        	if (object->choiceId == OBJECTSYNTAX_INTEGER_VALUE) {
            		ap->aclStatus = object->a.integer_value;
			return(OK);
        	} else {
                        errno = WRONGVALUE;
                        return(ERROR);
                }

    	default:
		errno = NOCREATION;
        	break;
    	}
    return(ERROR);
}


/*
 * -----------------------------------------------------------------------
 * Function:    ViewTableGet
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     Executes a GET operation on the managed object specified by
 *		oid and stores the result in the variable binding vbc
 *
 * Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * -----------------------------------------------------------------------
 */
	STATUS
ViewTableGet( AsnOid *oid, VarBindChoice *vbc)
{
	VField		field;
	struct ViewEntry	*vp;

	if (ViewMatch( oid, &field, &vp) != OK)
		return(ERROR);
	else	return(ViewTableFetch( field, vp, vbc->a.value));
}

/*
 * -----------------------------------------------------------------------
 * Function:    ViewMatch
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		VField			*field	(OUT)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     This routine searches the viewTable for an entry which
 *		matches oid. The found entry and field are returned
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS
ViewMatch( AsnOid *oid, VField *field, struct ViewEntry **vp)
{
	int			oidi;
	COMPARETYPE		res;
	struct ViewEntry	*pvp;
	AsnOid			*subtree;
	short			vindex;

	oidi = PARTYPREFIX;

	if ((*field = *(char *)(oid->octs + oidi++)) > VSTATUS)
		return(ERROR);
	vindex = *(char *)(oid->octs + oidi++);
	subtree = pNewOid( oid->octetLen - oidi);
        if ((oid->octs[ oidi] <= 2) && ( oid->octs[ oidi+1] < 40)) {
                subtree->octetLen = oid->octetLen - oidi-1;
                subtree->octs[0] = (oid->octs[oidi]*40) + oid->octs[oidi+1];
                memcpy( &subtree->octs[1], (oid->octs+(oidi+2)), subtree->octetLen-1);
        } else {
                subtree->octetLen = oid->octetLen - oidi + 1;
                memcpy( subtree->octs, (oid->octs + oidi), oid->octetLen);
        }
	pvp = PartyMIB.views;
	while (pvp) {
		if ( pvp->viewIndex > vindex)
			break;
		if (pvp->viewIndex == vindex)
			if (( res = CMPOID( subtree, pvp->viewSubtree)) == LESS)
				break;
			else	if (res == EQUAL) {
					*vp = pvp;
					FreeOid(subtree);
					return(OK);
				}
		pvp = pvp->next;
	}
	FreeOid(subtree);
	return(FAILED);
}

/*
 * -----------------------------------------------------------------------
 * Function:    ViewTableFetch
 *
 * Arguments:   VField			field	(IN)
 *		struct ViewEntry	*vp	(IN)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     Fetches the columnar object instance specified by field from
 *		the viewEntry specified by vp and stores the result in vbc
 *
 * Remarks:	Allocation of memory for values with variable size is done
 *		inside this routine
 *
 * -----------------------------------------------------------------------
 */
	STATUS
ViewTableFetch( VField field, ViewEntry *vp, ObjectSyntax *object)
{
	switch( field) {

	case VINDEX: /* not-accessible */
		return(FAILED);
	
	case VSUBTREE: /* not-accessible */
		object->choiceId = OBJECTSYNTAX_OBJECTID_VALUE;
		object->a.objectID_value = vp->viewSubtree;
		return(OK);

	case VMASK: /* read-create */
		object->choiceId = OBJECTSYNTAX_STRING_VALUE;
		if (vp->viewMask)
			object->a.string_value = vp->viewMask;
		else	object->a.string_value = &EmptyOctetstring;
		return(OK);

	case VTYPE: /* read-create */
		object->choiceId = OBJECTSYNTAX_INTEGER_VALUE;
		if (vp->viewType)
			object->a.integer_value = vp->viewType;
		else	object->a.integer_value = VIEWINCLUDED;
		return(OK);

	case VSTORAGETYPE: /* read-create */
		object->choiceId = OBJECTSYNTAX_INTEGER_VALUE;
		if (vp->viewStorageType)
			object->a.integer_value = vp->viewStorageType;
		else	object->a.integer_value = nonVolatile;
		return(OK);

	case VSTATUS: /* read-create */
		object->choiceId = OBJECTSYNTAX_INTEGER_VALUE;
		if (vp->viewStatus)
			object->a.integer_value = vp->viewStatus;
		else	object->a.integer_value = notReady;
		return(OK);
		
	default:
		break;
	}
	return(ERROR);
}


/*
 * ------------------------------------------------------------------------
 *
 *	Function:	ViewTableGetFirst
 *
 *	Arguments:	AsnOid			*oid	(IN)
 *			struct MibRecord	*mr	(IN)
 *			struct VarBindChoice	*vbc	(OUT)
 *
 *	Purpose:	Finds the lexicographically first managed object in the
 *			viewTable and stores the result in the variable binding
 *			vbc. If the table is empty, routines are called for the 
 *			lexicographically next object using mr.
 *
 *	Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * ------------------------------------------------------------------------
 */
	STATUS
ViewTableGetFirst( AsnOid *oid, VarBindChoice *vbc, struct MibRecord *mr)
{
	struct ViewEntry	*vp;
	int			oidi;
	VField			field;
	STATUS			status;

	vp = PartyMIB.views;
	field = VINDEX;
        if (!vp)
                if (mr->next)
                        return((mr->next->func)(oid, vbc, mr->next, P_GET_FIRST)
);
                else {
                        vbc->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
                        return(FAILED); /* end of mibview */
                }

	if ((oid->octs = (unsigned char *)realloc(oid->octs,
(mr->OID.octetLen+1)+vp->viewSubtree->octetLen)) == NULL)
		return(ERROR);
	oidi = mr->OID.octetLen;
	memcpy( oid->octs, mr->OID.octs, mr->OID.octetLen);
	*(oid->octs + oidi++) = (char) field;
	*(oid->octs + oidi++) = (char) vp->viewIndex;
	if (vp->viewSubtree->octs[0] >= 40)
		*(oid->octs + oidi++) = vp->viewSubtree->octs[0]/40;
	memcpy( (char *)(oid->octs + oidi), (char *)vp->viewSubtree->octs, vp->viewSubtree->octetLen);
	*(oid->octs + oidi) = *(oid->octs + oidi)%40;
	oid->octetLen = oidi + vp->viewSubtree->octetLen;
	while ((status = ViewTableFetch( field, vp, vbc->a.value)) == FAILED)
		/* if fetch fails this will fail for all instances of this field, try next field */
		if (++field <= VSTATUS)
			*(oid->octs + PARTYPREFIX) = (char) field;
		else	if (mr->next)
				return((mr->next->func)(oid, vbc, mr->next, P_GET_FIRST));
			else {
                                vbc->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
                                return(FAILED);
                        }
	return(status);
}

/*
 * ------------------------------------------------------------------------
 *
 *	Function:	ViewTableGetNext
 *
 *	Arguments:	AsnOid			*oid	(IN)
 *			struct MibRecord	*mr	(IN)
 *			struct VarBindChoice	*vbc	(OUT)
 *
 *	Purpose:	Executes a GETNEXT operation on the managed object specified
 *			by oid in the viewTable and stores the result in the variable binding
 *			vbc. If the table is empty, routines are called for the 
 *			lexicographically next object using mr.
 *
 *	Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * ------------------------------------------------------------------------
 */
	STATUS
ViewTableGetNext( AsnOid *oid, VarBindChoice *vbc, struct MibRecord *mr)
{
	struct ViewEntry	*vp;
	VField			field;
	short			vindex;
	AsnOid			*subtree;
	STATUS			status;
	int			oidi;

	field = VINDEX;
	if (ViewMatch( oid, &field, &vp) != OK) {
		oidi = PARTYPREFIX;
                if (oidi < oid->octetLen)
			field = (VField)*((char *)(oid->octs + oidi++)) ;
                if (oidi < oid->octetLen)
			vindex = *(char *)(oid->octs + oidi++);
		subtree = pNewOid(oid->octetLen - oidi);
                if (( ( oidi+1) < oid->octetLen) && ((oid->octs[oidi] <= 2) && ( oid->octs[oidi+1]
< 40)) ){
                        subtree->octetLen = (oid->octetLen - oidi)-1;
                        subtree->octs[0] = (oid->octs[oidi]*40) + oid->octs[oidi+1];
                        memcpy( &subtree->octs[1], (oid->octs+(oidi+2)), subtree->octetLen-1);
                } else {
                        subtree->octetLen = oid->octetLen - oidi;
                        memcpy( subtree->octs, (oid->octs + oidi), subtree->octetLen);
                }
                if (field < VINDEX) {
                        field = VINDEX;
                        subtree->octetLen = 0;
                }

		while (field <= VSTATUS) {
			vp = PartyMIB.views;
			while (vp) {
				if (vp->viewIndex > vindex)
					break;
				if (vp->viewIndex == vindex)
					if (CMPOID( subtree, vp->viewSubtree) == LESS)
						break;
				vp = vp->next;
			}
			if (vp)	break;
			field++;
			vindex = 0;
		}
		FreeOid(subtree);
		if (field > VSTATUS)
			if (mr->next)
				return(mr->next->func)(oid, vbc, mr->next, P_GET_FIRST);
			else {
                                vbc->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
                                return(FAILED);
                        }
	} else {
		vp = vp->next;
		while (!vp)
			if (++field > ASTATUS)
				if (mr->next)
					return(mr->next->func)(oid, vbc, mr->next, P_GET_FIRST);
				else {
                               		vbc->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
                                	return(FAILED);
                        	}
			else	vp = PartyMIB.views;
	}

	while (1) {
		if ((oid->octs = (unsigned char *)realloc( oid->octs,
(PARTYPREFIX+2)+vp->viewSubtree->octetLen)) == NULL)
			return(ERROR);
		oidi = PARTYPREFIX;
		*(oid->octs + oidi++) = (unsigned char) field;
		*(oid->octs + oidi++) = (unsigned char) vp->viewIndex;
		if (vp->viewSubtree->octs[0] >= 40)
			*(oid->octs + oidi++) = vp->viewSubtree->octs[0]/40;
		memcpy( (oid->octs + oidi), vp->viewSubtree->octs, vp->viewSubtree->octetLen);
		*(oid->octs + oidi) = *(oid->octs + oidi)%40;
		oid->octetLen = oidi + vp->viewSubtree->octetLen;
		if ((status = ( ViewTableFetch( field, vp, vbc->a.value))) != FAILED)
			return(status);
		if (++field > VSTATUS)
			if (mr->next)
				return(mr->next->func)(oid, vbc, mr->next, P_GET_FIRST);
			else {
                                vbc->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
                                return(FAILED);
                        }
		vp = PartyMIB.views;
	}
}

/*
 * -----------------------------------------------------------------------
 * Function:    ViewTableSet
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		struct VarBindChoice	*vbc	(IN)
 *
 * Purpose:     Executes a SET operation on the managed object specified by
 *		oid and uses the value in the variable binding vbc
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS
ViewTableSet( AsnOid *oid, VarBindChoice *vbc)
{
	VField	field;
	struct ViewEntry	*vp;

	if (ViewMatch( oid, &field, &vp) != OK)
		return(ERROR);
	else	return(ViewTableStore( field, vp, vbc));
}

/* 
 * -----------------------------------------------------------------------
 * Function:    ViewTableStore
 *
 * Arguments:   VField			field	(IN)
 *		struct ViewEntry	*vp	(IN)
 *		struct VarBindChoice	*vbc	(IN)
 *
 * Purpose:     Stores the value specified in the variable binding vbc in the
 *		managed object specified by its field and entry in the viewEntry vp.
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS
ViewTableStore( VField field, ViewEntry *vp, VarBindChoice *vbc)
{
	ObjectSyntax	*object;

	object = vbc->a.value;
	switch( field) {

	case VINDEX: /* not-accessible */
                errno = NOTWRITABLE;
                return(ERROR);
	
	case VSUBTREE: /* not-accessible */
                errno = NOTWRITABLE;
                return(ERROR);

	case VMASK: /* read-create */
		if (object->choiceId == OBJECTSYNTAX_STRING_VALUE) {
			if (!vp->viewMask)
				if (( vp->viewMask = pNewOct( object->a.string_value->octetLen)) ==
NULL) {
					errno = RESOURCEUNAVAILABLE;
					return(ERROR);
				}
			else if (( vp->viewMask->octs = (unsigned char *)realloc( vp->viewMask->octs,
object->a.string_value->octetLen)) == NULL) {
				errno = RESOURCEUNAVAILABLE;
				return(ERROR);
			}
			vp->viewMask->octetLen = object->a.string_value->octetLen;
			memcpy( vp->viewMask->octs, object->a.string_value->octs,
vp->viewMask->octetLen);
			return(OK);
		} else {
			errno = WRONGVALUE;
			return(ERROR);
		}

	case VTYPE: /* read-create */
		if ( object->choiceId == OBJECTSYNTAX_INTEGER_VALUE) {
			vp->viewType = object->a.integer_value;
			return(OK);
		} else {
			errno = WRONGVALUE;
			return(ERROR);
		}

	case VSTORAGETYPE: /* read-create */
		if ( object->choiceId == OBJECTSYNTAX_INTEGER_VALUE) {
			vp->viewStorageType = object->a.integer_value;
			return(OK);
		} else {
			errno = WRONGVALUE;
			return(ERROR);
		}

	case VSTATUS: /* read-create */
		if ( object->choiceId == OBJECTSYNTAX_INTEGER_VALUE) {
			vp->viewStatus = object->a.integer_value;
			return(OK);
		} else {
			errno = WRONGVALUE;
			return(ERROR);
		}
		
	default:
		errno = NOCREATION;
		break;
	}
	return(ERROR);
}
