/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:    spmPartyINT.c
 *
 * Date:        May 1994
 *
 * Purpose:     This file contains the partymib interface.
 *
 * Author( s):  Koert W.G. Flapper
 *
 * -----------------------------------------------------------------------
 */

#include <spmPartyMIB.h>

static AclEntry *_acltable;

	

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetPartyIndex
 *
 * Arguments:   partyId		OID	(in)
 *		partyIndex	AsnInt	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the party's index identified 
 * 		by  party identifier 
 *
 * Remarks:	none
 *
 *-------------------------------------------------------------------
 */
	STATUS	
MibGetPartyIndex( UAsnInt index, short *pindex )
{ 
	PartyEntry	*pe;

	pe = (PartyEntry *)index;
	*pindex = pe->partyIndex;
	return(OK);
}		

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetPartyID
 *
 * Arguments:   index		UAsnInt	(in)
 *		pid		*AsnOid	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the party identifier identified 
 *              by index 
 *
 * Remarks:	none
 *
 *-------------------------------------------------------------------
 */
	STATUS
MibGetPartyID( UAsnInt index, AsnOid *pid )
{ 
	PartyEntry	*pe;

	pe = (PartyEntry *)index;
	pid->octetLen = pe->partyIdentity->octetLen;
	memcpy( pid->octs, pe->partyIdentity->octs, pid->octetLen);
	return(OK);
}		


/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetPartyTDomain
 *
 * Arguments:   index		UAsnInt	(in)
 *		partyTDomain	*AsnOid	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the party's transport domain identified 
 *              by  party identifier 
 *
 * Remarks:	none
 *
 *-------------------------------------------------------------------
 */
	STATUS
MibGetPartyTDomain( UAsnInt index, AsnOid *pd )
{ 
	PartyEntry	*pe;

	pe = (PartyEntry *)index;
	pd->octetLen = pe->partyTDomain->octetLen;
	memcpy( pd->octs, pe->partyTDomain->octs, pd->octetLen);
	return(OK);
}		


/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetPartyTAddress
 *
 * Arguments:   partyIndex	UAsnInt		(in)
 *		partyTAddress	*AsnOcts	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the party's transport address identified 
 *              by  party identifier 
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibGetPartyTAddress( UAsnInt index, AsnOcts *pa )
{ 
	PartyEntry	*pe;

	pe = (PartyEntry *)index;
	pa->octetLen = pe->partyTAddress->octetLen;
	memcpy( pa->octs, pe->partyTAddress->octs, pa->octetLen);
	return(OK);
}				



/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetPartyMaxMessageSize
 *
 * Arguments:   partyIndex		UAsnInt		(in)
 *		partyMaxMessageSize	*AsnInt	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the party's maximum message size which is 
 *              supported by this entity identified by  party identifier 
 *
 * Remarks:	none
 *
 * -------------------------------------------------------------------
 */
	STATUS
MibGetPartyMaxMessageSize( UAsnInt index, AsnInt *mms )
{
	PartyEntry	*pe;

	pe = (PartyEntry *)index;
	*mms = pe->partyMaxMessageSize;
	return(OK);
}		



/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetPartyLocal
 *
 * Arguments:   index		UAsnInt		(in)
 *		partyLocal	*Truthvalue	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the indication of whether the party referenced
 *              by index is locally realized
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibGetPartyLocal( UAsnInt index, TruthValue *pl )
{ 
	PartyEntry	*pe;

	pe = (PartyEntry *)index;
	*pl = pe->partyLocal;
	return(OK);
}		



/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetPartyAuthProtocol
 *
 * Arguments:   index		UAsnInt		(in)
 *		*partyAuthProtocol	*AsnOid	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the used authentication algorithm identified 
 *              by  party identifier 
 *
 * -------------------------------------------------------------------
 */
	STATUS
MibGetPartyAuthProtocol( UAsnInt index, AsnOid *pap )
{ 
	PartyEntry	*pe;

	pe = (PartyEntry *)index;
	pap->octetLen = pe->partyAuthProtocol->octetLen;
	memcpy( pap->octs, pe->partyAuthProtocol->octs, pap->octetLen);
	return(OK);
}
	

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetPartyAuthClock
 *
 * Arguments:   index		UAsnInt		(in)
 *		partyAuthClock	*UAsnInt	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the current time for the party identified 
 *              by  party identifier 
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibGetPartyAuthClock( UAsnInt index, Clock *pac )
{ 
	PartyEntry	*pe;

	pe = (PartyEntry *)index;
	*pac = pe->partyAuthClock;
	return(OK);
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetPartyAuthPrivate
 *
 * Arguments:   index		UAsnInt		(in)
 *		partyAuthPrivate	*AsnOcts	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the private authentication key identified 
 *              by  party identifier 
 *
 * -------------------------------------------------------------------
 */
	STATUS
MibGetPartyAuthPrivate( UAsnInt index, AsnOcts *papr )
{ 
	PartyEntry	*pe;

	pe = (PartyEntry *)index;
	if (!pe->partyAuthPrivate)
		return(FAILED);
	papr->octetLen = pe->partyAuthPrivate->octetLen;
	memcpy( papr->octs, pe->partyAuthPrivate->octs, papr->octetLen);
	return(OK);
}
		
/*
 *-------------------------------------------------------------------
 * Function:    MibGetPartyAuthPublic
 * Arguments:   index		UAsnInt		(in)
 *		partyAuthPublic	AsnOcts	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the public authentication key identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

STATUS	MibGetPartyAuthPublic( UAsnInt index, AsnOcts *papu)
{
        PartyEntry      *pe;

        pe = (PartyEntry *)index;
	if (!pe->partyAuthPublic)
		return(FAILED);
        papu->octetLen = pe->partyAuthPublic->octetLen;
        memcpy( papu->octs, pe->partyAuthPublic->octs, papu->octetLen);
	return(OK);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetPartyAuthLifetime
 *
 * Arguments:   index		UAsnInt		(in)
 *		partyAuthLifetime	*AsnInt	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the upperbound of acceptable delivery delay 
 *              for the party identified by  party identifier 
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibGetPartyAuthLifetime( UAsnInt index, AsnInt *palt )
{ 
	PartyEntry	*pe;

	pe = (PartyEntry *)index;
	*palt = pe->partyAuthLifetime;
	return(OK);
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetPartyPrivProtocol
 *
 * Arguments:   index		UAsnInt		(in)
 *		partyPrivProtocol	*AsnOid	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the supported privacy protocol for the party 
 *              identified by  party identifier 
 *
 * -------------------------------------------------------------------
 */
	STATUS
MibGetPartyPrivProtocol( UAsnInt index, AsnOid *ppp )
{ 
	PartyEntry	*pe;

	pe = (PartyEntry *)index;
	ppp->octetLen = pe->partyPrivProtocol->octetLen;
	memcpy( ppp->octs, pe->partyPrivProtocol->octs, ppp->octetLen);
	return(OK);
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetPartyPrivPrivate
 *
 * Arguments:   index		UAsnInt		(in)
 *		partyPrivPrivate	*AsnOcts	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the private privacy key identified 
 *              by  party identifier 
 *
 * -------------------------------------------------------------------
 */
	STATUS
MibGetPartyPrivPrivate( UAsnInt index, AsnOcts *pppr )
{ 
	PartyEntry	*pe;

	pe = (PartyEntry *)index;
	if (!pe->partyPrivPrivate)
		return(FAILED);
	pppr->octetLen = pe->partyPrivPrivate->octetLen;
	memcpy( pppr->octs, pe->partyPrivPrivate->octs, pppr->octetLen);
	return(OK);
}
		
/*
 *-------------------------------------------------------------------
 * Function:    MibGetPartyPrivPublic
 * Arguments:   index		UAsnInt		(in)
 *		partyPrivPublic	AsnOcts	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the public privacy key identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

STATUS	MibGetPartyPrivPublic( UAsnInt index, AsnOcts *pppu)
{
        PartyEntry      *pe;

        pe = (PartyEntry *)index;
	if (!pe->partyPrivPublic)
		return(FAILED);
        pppu->octetLen = pe->partyPrivPublic->octetLen;
        memcpy( pppu->octs, pe->partyPrivPublic->octs, pppu->octetLen);
	return(OK);
}


/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetPartyCloneFrom
 *
 * Arguments:   index		UAsnInt		(in)
 *		pcf		*AsnOid	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the party from which the party 
 *              identified by  index is cloned from
 *
 * -------------------------------------------------------------------
 */
	STATUS
MibGetPartyCloneFrom( UAsnInt index, AsnOid *pcf )
{ 
	PartyEntry	*pe;

	pe = (PartyEntry *)index;
	if ( !pe->partyCloneFrom)
		return(FAILED);
	pcf->octetLen = pe->partyCloneFrom->octetLen;
	memcpy( pcf->octs, pe->partyCloneFrom->octs, pcf->octetLen);
	return(OK);
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetPartyStorageType
 *
 * Arguments:   index	UAsnInt		(in)
 *		pst	StorageType	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the partyStorageType referenced by  
 *              index
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibGetPartyStorageType( UAsnInt index, StorageType *pst )
{  
	PartyEntry	*pe;

	pe = (PartyEntry *)index;
	*pst = pe->partyStorageType;
	return(OK);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetPartyStatus
 *
 * Arguments:   index	UAsnInt		(in)
 *		prs	RowStatus	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the partyStatus referenced by  
 *              index
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibGetPartyStatus( UAsnInt index, RowStatus *prs )
{  
	PartyEntry	*pe;

	pe = (PartyEntry *)index;
	*prs = pe->partyStatus;
	return(OK);
}


/*
 *-------------------------------------------------------------------
 * Function:    MibPutPartyTDomain
 * Arguments:   index		UAsnInt	(in)
 *		partyTDomain	AsnOid	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the party's transport domain identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

STATUS	MibPutPartyTDomain( UAsnInt index, AsnOid pd )
{
        PartyEntry      *pe;

        pe = (PartyEntry *)index;
        pe->partyTDomain->octetLen = pd.octetLen;
        memcpy( pe->partyTDomain->octs, pd.octs, pd.octetLen);
	return(OK);
}

/*
 *-------------------------------------------------------------------
 * Function:    MibPutPartyTAddress
 * Arguments:   index		UAsnInt		(in)
 *		partyTAddress	AsnOct	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the party's transport address identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

STATUS	MibPutPartyTAddress( UAsnInt index, AsnOcts pa)
{
        PartyEntry      *pe;

        pe = (PartyEntry *)index;
        pe->partyTAddress->octetLen = pa.octetLen;
        memcpy( pe->partyTAddress->octs, pa.octs, pa.octetLen);
	return(OK);
}

/*
 *-------------------------------------------------------------------
 * Function:    MibPutPartyMaxMessageSize
 * Arguments:   index		UAsnInt		(in)
 *		partyMaxMessageSize	AsnInt	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the party's maximum message size which is 
 *              supported by this entity identified by  party identifier 
 *-------------------------------------------------------------------
 */

STATUS	MibPutPartyMaxMessageSize( UAsnInt index, UAsnInt pmms)
{
        PartyEntry      *pe;

        pe = (PartyEntry *)index;
        pe->partyMaxMessageSize = pmms;
	return(OK);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibPutPartyLocal
 *
 * Arguments:   index		UAsnInt		(in)
 *		partyLocal	Truthvalue	(in) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Stores the indication of whether the party referenced
 *              by index is locally realized
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibPutPartyLocal( UAsnInt index, TruthValue pl )
{ 
        PartyEntry      *pe;

        pe = (PartyEntry *)index;
        pe->partyLocal = pl;
	return(OK);
}		

/*
 *-------------------------------------------------------------------
 * Function:    MibPutPartyAuthProtocol
 * Arguments:   index		UAsnInt		(in)
 *		partyAuthProtocol	AsnOid	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the used authentication algorithm identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

STATUS	MibPutPartyAuthProtocol( UAsnInt index, AsnOid pap)
{
        PartyEntry      *pe;

        pe = (PartyEntry *)index;
        pe->partyTAddress->octetLen = pap.octetLen;
        memcpy( pe->partyTAddress->octs, pap.octs, pap.octetLen);
	return(OK);
}


/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibPutPartyAuthClock
 *
 * Arguments:   index		UAsnInt		(in)
 *		partyAuthClock	AsnInt		(in) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Stores the current time for the party identified 
 *              by  party identifier 
 *
 * -------------------------------------------------------------------
 */
	STATUS
MibPutPartyAuthClock( UAsnInt index, AsnInt pac )
{ 
        PartyEntry      *pe;

        pe = (PartyEntry *)index;
        pe->partyAuthClock = pac;
	return(OK);
}

/*
 *-------------------------------------------------------------------
 * Function:    MibPutPartyAuthPrivate
 * Arguments:   index		UAsnInt		(in)
 *		partyAuthPrivate	AsnOcts	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the private authentication key identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

STATUS	MibPutPartyAuthPrivate( UAsnInt index, AsnOcts papr)
{
        PartyEntry      *pe;

        pe = (PartyEntry *)index;
	if (!pe->partyAuthPrivate)
		pe->partyAuthPrivate = (AsnOcts *)pNewOct(papr.octetLen);
        pe->partyAuthPrivate->octetLen = papr.octetLen;
        memcpy( pe->partyAuthPrivate->octs, papr.octs, papr.octetLen);
	return(OK);
}

/*
 *-------------------------------------------------------------------
 * Function:    MibPutPartyAuthPublic
 * Arguments:   index		UAsnInt		(in)
 *		partyAuthPublic	unsigned charSTRING	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the public authentication key identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

STATUS	MibPutPartyAuthPublic( UAsnInt index, AsnOcts papu)
{
        PartyEntry      *pe;

        pe = (PartyEntry *)index;
	if (!pe->partyAuthPublic)
		pe->partyAuthPublic = (AsnOcts *)pNewOct(papu.octetLen);
        pe->partyAuthPublic->octetLen = papu.octetLen;
        memcpy( pe->partyAuthPublic->octs, papu.octs, papu.octetLen);
	return(OK);
}

/*
 *-------------------------------------------------------------------
 * Function:    MibPutPartyAuthLifetime
 * Arguments:   index		UAsnInt		(in)
 *		partyAuthLifetime	AsnInt	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the upperbound of acceptable delivery delay 
 *              for the party identified by  party identifier 
 *-------------------------------------------------------------------
 */

STATUS	MibPutPartyAuthLifetime( UAsnInt index, AsnInt palt)
{
        PartyEntry      *pe;

        pe = (PartyEntry *)index;
        pe->partyAuthLifetime = palt;
	return(OK);
}

/*
 *-------------------------------------------------------------------
 * Function:    MibPutPartyPrivProtocol
 * Arguments:   index		UAsnInt		(in)
 *		partyPrivProtocol	AsnOid	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the supported privacy protocol for the party 
 *              identified by  party identifier 
 *-------------------------------------------------------------------
 */

STATUS	MibPutPartyPrivProtocol( UAsnInt index, AsnOid ppp)
{
        PartyEntry      *pe;

        pe = (PartyEntry *)index;
        pe->partyPrivProtocol->octetLen = ppp.octetLen;
        memcpy( pe->partyPrivProtocol->octs, ppp.octs, ppp.octetLen);
	return(OK);
}

/*
 *-------------------------------------------------------------------
 * Function:    MibPutPartyPrivPrivate
 * Arguments:   index		UAsnInt		(in)
 *		partyPrivPrivate	AsnOcts	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the private privacy key identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

STATUS	MibPutPartyPrivPrivate( UAsnInt index, AsnOcts pppr)
{
        PartyEntry      *pe;

        pe = (PartyEntry *)index;
	if (!pe->partyPrivPrivate)
		pe->partyPrivPrivate = (AsnOcts *)pNewOct(pppr.octetLen);
        pe->partyPrivPrivate->octetLen = pppr.octetLen;
        memcpy( pe->partyPrivPrivate->octs, pppr.octs, pppr.octetLen);
	return(OK);
}

/*
 *-------------------------------------------------------------------
 * Function:    MibPutPartyPrivPublic
 * Arguments:   index		UAsnInt		(in)
 *		partyPrivPublic	AsnOcts	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the public privacy key identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

STATUS	MibPutPartyPrivPublic( UAsnInt index, AsnOcts pppu)
{
        PartyEntry      *pe;

        pe = (PartyEntry *)index;
	if (!pe->partyPrivPublic)
		pe->partyPrivPublic = (AsnOcts *)pNewOct(pppu.octetLen);
        pe->partyPrivPublic->octetLen = pppu.octetLen;
        memcpy( pe->partyPrivPublic->octs, pppu.octs, pppu.octetLen);
	return(OK);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibPutPartyCloneFrom
 *
 * Arguments:   index		UAsnInt		(in)
 *		pcf		AsnOid		(in) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Stores the party from which the party 
 *              identified by  index is cloned from
 *
 * -------------------------------------------------------------------
 */
	STATUS
MibPutPartyCloneFrom( UAsnInt index, AsnOid pcf )
{ 
	PartyEntry	*pe;

	pe = (PartyEntry *)index;
	if (!pe->partyCloneFrom)
		pe->partyCloneFrom = (AsnOid *)pNewOid(pcf.octetLen);
	pe->partyCloneFrom->octetLen = pcf.octetLen;
	memcpy( pe->partyCloneFrom->octs, pcf.octs, pcf.octetLen);
	return(OK);
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibPutPartyStorageType
 *
 * Arguments:   index	UAsnInt		(in)
 *		pst	StorageType	(in) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Stores the partyStorageType referenced by  
 *              index
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibPutPartyStorageType( UAsnInt index, StorageType pst )
{  
	PartyEntry	*pe;

	pe = (PartyEntry *)index;
	pe->partyStorageType = pst;
	return(OK);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibPutPartyStatus
 *
 * Arguments:   index	UAsnInt		(in)
 *		prs	RowStatus	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Stores the partyStatus referenced by  
 *              index
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibPutPartyStatus( UAsnInt index, RowStatus prs )
{  
	PartyEntry	*pe;

	pe = (PartyEntry *)index;
	pe->partyStatus = prs;
	return(OK);
}


/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetContextIndex
 *
 * Arguments:   contextId		UAsnInt	(in)
 *		contextIndex	AsnInt	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the context's index identified 
 *              by  context identifier 
 *
 * -------------------------------------------------------------------
 */
	STATUS
MibGetContextIndex( UAsnInt index, short *cindex )
{ 
	ContextEntry	*ce;

	ce = (ContextEntry *)index;
	*cindex = ce->contextIndex;
	return(OK);
}		

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetContextID
 *
 * Arguments:   contextId	UAsnInt		(in)
 *		cid		*AsnOid		(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the contextidentity referenced by  
 *              index
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibGetContextID( UAsnInt index, AsnOid *cid )
{  
	ContextEntry	*ce;

	ce = (ContextEntry *)index;
	cid->octetLen = ce->contextIdentity->octetLen;
	memcpy( cid->octs, ce->contextIdentity->octs, cid->octetLen);
	return(OK);
}


/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetContextLocal
 *
 * Arguments:   contextId	UAsnInt		(in)
 *		contextLocal	*Truthvalue	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the indication of whether the context referenced
 *              by  contextId is locally realized
 *
 * -------------------------------------------------------------------
 */
	STATUS
MibGetContextLocal( UAsnInt index, TruthValue *cl )
{ 
	ContextEntry	*ce;

	ce = (ContextEntry *)index;
	*cl = ce->contextLocal;
	return(OK);
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetContextViewIndex
 *
 * Arguments:   contextId	UAsnInt		(in)
 *		index		*AsnInt	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the viewIndex referenced by  contextId
 *
 * -------------------------------------------------------------------
 */
	STATUS
MibGetContextViewIndex( UAsnInt	index, AsnInt *cvi )
{ 
	ContextEntry	*ce;

	ce = (ContextEntry *)index;
	*cvi = ce->contextLocal;
	return(OK);
}		



/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetContextLocalEntity
 *
 * Arguments:   contextId	UAsnInt		(in)
 *		contextLocalEntity	*AsnOcts	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the contextLocalEntity referenced by  
 *              contextId
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibGetContextLocalEntity( UAsnInt index, AsnOcts *cle )
{ 
	ContextEntry	*ce;

	ce = (ContextEntry *)index;
	if (ce->contextLocalEntity == NULL) {
		cle->octetLen = 0 ;
		cle->octs[0] = '\0';
		return FAILED ;
	} else {
		cle->octetLen = ce->contextLocalEntity->octetLen;
		memcpy(cle->octs, ce->contextLocalEntity->octs, cle->octetLen);
		return OK ;
	}
 }



/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetContextLocalTime
 *
 * Arguments:   contextId	UAsnInt		(in)
 *		contextLocalTime	AsnOcts	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the contextLocalTime referenced by  
 *              contextId
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibGetContextLocalTime( UAsnInt index, AsnOid *clt )
{  
	ContextEntry	*ce;

	ce = (ContextEntry *)index;
	clt->octetLen = ce->contextLocalTime->octetLen;
	memcpy( clt->octs, ce->contextLocalTime->octs, clt->octetLen);
	return(OK);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetContextProxyDstParty
 *
 * Arguments:   index	UAsnInt		(in)
 *		contextProxyDstParty	AsnOid	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the contextProxyDstParty referenced by  
 *              index
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibGetContextProxyDstParty( UAsnInt index, AsnOid *cpdp )
{  
	ContextEntry	*ce;

	ce = (ContextEntry *)index;
	if (!ce->contextProxyDstParty)
		return(FAILED);
	cpdp->octetLen = ce->contextProxyDstParty->octetLen;
	memcpy(cpdp->octs, ce->contextProxyDstParty->octs,
cpdp->octetLen);
	return(OK);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetContextProxySrcParty
 *
 * Arguments:   index	UAsnInt		(in)
 *		contextProxySrcParty	AsnOid	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the contextProxySrcParty referenced by  
 *              index
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibGetContextProxySrcParty( UAsnInt index, AsnOid *cpsp )
{  
	ContextEntry	*ce;

	ce = (ContextEntry *)index;
	if (!ce->contextProxySrcParty)
		return(FAILED);
	cpsp->octetLen = ce->contextProxySrcParty->octetLen;
	memcpy(cpsp->octs, ce->contextProxySrcParty->octs,
cpsp->octetLen);
	return(OK);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetContextProxyContext
 *
 * Arguments:   index	UAsnInt		(in)
 *		contextProxyContext	AsnOid	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the contextProxyContext referenced by  
 *              index
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibGetContextProxyContext( UAsnInt index, AsnOid *cpc )
{  
	ContextEntry	*ce;

	ce = (ContextEntry *)index;
	if (!ce->contextProxyContext)
		return(FAILED);
	cpc->octetLen = ce->contextProxyContext->octetLen;
	memcpy(cpc->octs, ce->contextProxyContext->octs,
cpc->octetLen);
	return(OK);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetContextStorageType
 *
 * Arguments:   index	UAsnInt		(in)
 *		cst	StorageType	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the contextStorageType referenced by  
 *              index
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibGetContextStorageType( UAsnInt index, StorageType *cst )
{  
	ContextEntry	*ce;

	ce = (ContextEntry *)index;
	*cst = ce->contextStorageType;
	return(OK);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetContextStatus
 *
 * Arguments:   index	UAsnInt		(in)
 *		crs	RowStatus	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the contextStatus referenced by  
 *              index
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibGetContextStatus( UAsnInt index, RowStatus *crs )
{  
	ContextEntry	*ce;

	ce = (ContextEntry *)index;
	*crs = ce->contextStatus;
	return(OK);
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibPutContextLocal
 *
 * Arguments:   contextId	UAsnInt		(in)
 *		contextLocal	Truthvalue	(in) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Stores the indication of whether the context referenced
 *              by  contextId is locally realized
 *
 * -------------------------------------------------------------------
 */
	STATUS
MibPutContextLocal( UAsnInt index, TruthValue cl )
{ 
	ContextEntry	*ce;

	ce = (ContextEntry *)index;
	ce->contextLocal = cl;
	return(OK);
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibPutContextViewIndex
 *
 * Arguments:   contextId	UAsnInt		(in)
 *		index		AsnInt	(in) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Stores the viewIndex referenced by  contextId
 *
 * -------------------------------------------------------------------
 */
	STATUS
MibPutContextViewIndex( UAsnInt	index, AsnInt cvi )
{ 
	ContextEntry	*ce;

	ce = (ContextEntry *)index;
	ce->contextLocal = cvi;
	return(OK);
}		



/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibPutContextLocalEntity
 *
 * Arguments:   contextId	UAsnInt		(in)
 *		contextLocalEntity	*AsnOcts	(in) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Stores the contextLocalEntity referenced by  
 *              contextId
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibPutContextLocalEntity( UAsnInt index, AsnOcts cle )
{ 
	ContextEntry	*ce;

	ce = (ContextEntry *)index;
	if (ce->contextLocalEntity == NULL )
		ce->contextLocalEntity = (AsnOcts *)pNewOct(cle.octetLen);
	ce->contextLocalEntity->octetLen = cle.octetLen;
	memcpy( ce->contextLocalEntity->octs, cle.octs, cle.octetLen);
	return OK ;
 }



/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibPutContextLocalTime
 *
 * Arguments:   contextId	UAsnInt		(in)
 *		contextLocalTime	AsnOcts	(in) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Stores the contextLocalTime referenced by  
 *              contextId
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibPutContextLocalTime( UAsnInt index, AsnOid clt )
{  
	ContextEntry	*ce;

	ce = (ContextEntry *)index;
	ce->contextLocalTime->octetLen = clt.octetLen;
	memcpy( ce->contextLocalTime->octs, clt.octs, clt.octetLen);
	return(OK);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibPutContextProxyDstParty
 *
 * Arguments:   index	UAsnInt		(in)
 *		contextProxyDstParty	AsnOid	(in) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Stores the contextProxyDstParty referenced by  
 *              index
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibPutContextProxyDstParty( UAsnInt index, AsnOid cpdp )
{  
	ContextEntry	*ce;

	ce = (ContextEntry *)index;
	if (!ce->contextProxyDstParty)
		ce->contextProxyDstParty = (AsnOid *)pNewOid(cpdp.octetLen);
	ce->contextProxyDstParty->octetLen = cpdp.octetLen;
	memcpy(ce->contextProxyDstParty->octs, cpdp.octs, cpdp.octetLen);
	return(OK);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibPutContextProxySrcParty
 *
 * Arguments:   index	UAsnInt		(in)
 *		contextProxySrcParty	AsnOid	(in) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Stores the contextProxySrcParty referenced by  
 *              index
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibPutContextProxySrcParty( UAsnInt index, AsnOid cpsp )
{  
	ContextEntry	*ce;

	ce = (ContextEntry *)index;
	if (!ce->contextProxySrcParty)
		ce->contextProxySrcParty = (AsnOid *)pNewOid(cpsp.octetLen);
	ce->contextProxySrcParty->octetLen = cpsp.octetLen;
	memcpy(ce->contextProxySrcParty->octs, cpsp.octs, cpsp.octetLen);
	return(OK);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibPutContextProxyContext
 *
 * Arguments:   index	UAsnInt		(in)
 *		contextProxyContext	AsnOid	(in) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Stores the contextProxyContext referenced by  
 *              index
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibPutContextProxyContext( UAsnInt index, AsnOid cpc )
{  
	ContextEntry	*ce;

	ce = (ContextEntry *)index;
	if (!ce->contextProxyContext)
		ce->contextProxyContext = (AsnOid *)pNewOid(cpc.octetLen);
	ce->contextProxyContext->octetLen = cpc.octetLen;
	memcpy(ce->contextProxyContext->octs, cpc.octs, cpc.octetLen);
	return(OK);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibPutContextStorageType
 *
 * Arguments:   index	UAsnInt		(in)
 *		cst	StorageType	(in) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Stores the contextStorageType referenced by  
 *              index
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibPutContextStorageType( UAsnInt index, StorageType cst )
{  
	ContextEntry	*ce;

	ce = (ContextEntry *)index;
	ce->contextStorageType = cst;
	return(OK);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibPutContextStatus
 *
 * Arguments:   index	UAsnInt		(in)
 *		crs	RowStatus	(in) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Stores the contextStatus referenced by  
 *              index
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibPutContextStatus( UAsnInt index, RowStatus crs )
{  
	ContextEntry	*ce;

	ce = (ContextEntry *)index;
	ce->contextStatus = crs;
	return(OK);
}


/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibResetAclEntry
 *
 * Arguments:   
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Resets the pointer _acltable
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibResetAclEntry(  )
{
	_acltable = PartyMIB.acls;
     	return(OK);
 }



/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetAclEntry
 *
 * Arguments:   target		*UAsnInt		(out)
 *		subject		*UAsnInt		(out)
 *		resources	*UAsnInt		(out)
 *		aclPrivileges	*AsnInt	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Gets the acl-entry pointed to by _acltable
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibGetAclEntry( UAsnInt *subject, UAsnInt *target, UAsnInt *resources,
	   AsnInt *aclprivileges )
{  
	if (_acltable) {
		*subject = _acltable->aclSubject;
		*target = _acltable->aclTarget;
		*resources = _acltable->aclResources;
		*aclprivileges = _acltable->aclPrivileges;
		_acltable = _acltable->next;
		return(OK);
	} else 
		return(FAILED);
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetAcl
 *
 * Arguments:   target		*UAsnInt		(in/out)
 *		subject		*UAsnInt		(in/out)
 *		resources	*UAsnInt		(in/out)
 *		aclPrivileges	*AsnInt	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the acl entry identified by the combined input 
 *              parametervalues. If inputparameters are NULL they are
 *		irrelevant.
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibGetAcl( UAsnInt *target, UAsnInt *subject, UAsnInt *resources,
	   AsnInt *aclprivileges )
{
	TruthValue	targetok = false, subjectok = false,
			resourcesok = false;
	AclEntry	*at;

        if (*target == 0)
                targetok = true;
        if (*subject == 0)
                subjectok = true;
        if (*resources == 0)
                resourcesok = true;
	at = PartyMIB.acls;
	while (at) {
		if((( at->aclTarget == *target) ||
( targetok == true))  && ((at->aclSubject == *subject) ||
( subjectok == true)) && (( at->aclResources == *resources) || ( resourcesok == true) ) )
		    	{ 
				*aclprivileges = at->aclPrivileges;
				if (targetok == true)
					*target = at->aclTarget;
				if (subjectok == true)
					*subject = at->aclSubject;
				if (resourcesok == true)
					*resources = at->aclResources;
		      		return(OK);
		    	}
	      	at = at->next;
	}
	return(FAILED);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetAclStorageType
 *
 * Arguments:   target		*UAsnInt		(in/out)
 *		subject		*UAsnInt		(in/out)
 *		resources	*UAsnInt		(in/out)
 *		ast		*StorageType		(out)
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the aclStorageType referenced by  
 *              index
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibGetAclStorageType( UAsnInt *target, UAsnInt *subject, UAsnInt *resources, StorageType *ast )
{  
	AclEntry	*at;

	at = PartyMIB.acls;
	while (at) {
		if( ( at->aclTarget == *target) && (at->aclSubject == *subject) && (
at->aclResources == *resources) ) {
			*ast = at->aclStorageType;
			return(OK);
		}
		at = at->next;
	}
	return(FAILED);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetAclStatus
 *
 * Arguments:   target		*UAsnInt		(in/out)
 *		subject		*UAsnInt		(in/out)
 *		resources	*UAsnInt		(in/out)
 *		prs	RowStatus	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the aclStatus referenced by  
 *              index
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibGetAclStatus(UAsnInt *target, UAsnInt *subject, UAsnInt *resources, RowStatus *ars )
{  
	AclEntry	*at;

	at = PartyMIB.acls;
	while (at) {
		if( ( at->aclTarget == *target) && (at->aclSubject == *subject) && ( at->aclResources == *resources) ) {
			*ars = at->aclStatus;
			return(OK);
		}
		at = at->next;
	}
	return(FAILED);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibPutAcl
 *
 * Arguments:   target		UAsnInt		(in)
 *		subject		UAsnInt		(in)
 *		resources	UAsnInt		(in)
 *		aclPrivileges	AsnInt			(in) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Stores the acl entry identified by the combined input 
 *              parametervalues. 
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibPutAcl( UAsnInt target, UAsnInt subject, UAsnInt resources,
	   AsnInt aclprivileges )
{
	AclEntry	*at;

	at = PartyMIB.acls;
	while (at) {
		if( ( at->aclTarget == target) && (at->aclSubject == subject) && 
( at->aclResources == resources) ) { 
			at->aclPrivileges = aclprivileges;
		      	return(OK);
		}
	      	at = at->next;
	}
	return(FAILED);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibPutAclStorageType
 *
 * Arguments:   target		UAsnInt		(in)
 *		subject		UAsnInt		(in)
 *		resources	UAsnInt		(in)
 *		pst		StorageType	(in) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Stores the aclStorageType referenced by the 
 *              indices
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibPutAclStorageType( UAsnInt target, UAsnInt subject, UAsnInt resources, StorageType ast )
{  
	AclEntry	*at;

	at = PartyMIB.acls;
	while (at) {
		if( ( at->aclTarget == target) && (at->aclSubject == subject) && ( at->aclResources == resources) ) {
			at->aclStorageType = ast;
			return(OK);
		}
		at = at->next;
	}
	return(FAILED);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibPutAclStatus
 *
 * Arguments:   target		UAsnInt		(in)
 *		subject		UAsnInt		(in)
 *		resources	UAsnInt		(in)
 *		prs		RowStatus	(in) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Stores the partyStatus referenced by the 
 *              indices
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibPutAclStatus(UAsnInt target, UAsnInt subject, UAsnInt resources, RowStatus ars )
{  
	AclEntry	*at;

	at = PartyMIB.acls;
	while (at) {
		if( ( at->aclTarget == target) && (at->aclSubject == subject) && ( at->aclResources == resources) ) {
			at->aclStorageType = ars;
			return(OK);
		}
		at = at->next;
	}
	return(FAILED);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetViewStatus
 *
 * Arguments:   index	UAsnInt	(in)
 *		prs	*RowStatus	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the aclStatus referenced by  
 *              index
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibGetViewStatus(UAsnInt index, RowStatus *vrs )
{  
	ViewEntry	*vt;
	UAsnInt		viewindex;

	MibGetContextViewIndex( index, &viewindex);

	vt = PartyMIB.views;
	while (vt) {
		if( vt->viewIndex == viewindex ) {
			*vrs = vt->viewStatus;
			return(OK);
		}
		vt = vt->next;
	}
	return(FAILED);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibGetViewStorageType
 *
 * Arguments:   index	UAsnInt	(in)
 *		prs	*StorageType	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the storagetype referenced by  
 *              index
 *
 * -------------------------------------------------------------------
 */
	STATUS	
MibGetViewStorageType(UAsnInt index, StorageType *vst )
{  
	ViewEntry	*vt;
	UAsnInt		viewindex;

	MibGetContextViewIndex( index, &viewindex);

	vt = PartyMIB.views;
	while (vt) {
		if( vt->viewIndex == viewindex ) {
			*vst = vt->viewStorageType;
			return(OK);
		}
		vt = vt->next;
	}
	return(FAILED);
}

/* -------------------------------------------------------------------
 */
