/* -----------------------------------------------------------------------
 * $Id: spmDES.c,v 1.1 1994/03/30 14:04:35 snmp Exp $
 * Project:     UT-snmpV2
 * Filename:    spmDES.c
 * Date:        Januari 1994
 * Purpose:     Interface to DES.
 * Author(s):   Rene Oude Vrielink
 * -----------------------------------------------------------------------
 */

#include <stdio.h>
#include <spmDES.h>
#define BUFSIZE ((long)1024*8)

/* -----------------------------------------------------------------------
 * Function:    DesEncrypt 
 * Arguments:   privateKey,    Private key (IN)
 *		privData,    the (encrypted) data. (IN/OUT).
 * Purpose:     Encrypts privData according to specified protocol,
 *              as specified in RFC 1446 (Security Protocols for SNMPv2)
 *              section 1.5.2.
 * Remarks:
 * -----------------------------------------------------------------------
 */
	STATUS
DesEncrypt( AsnOcts *privateKey, AsnOcts *privData )
{
	int	rem,i;
	des_key_schedule schedule;
	AsnOcts	*newPrivData = NewOct( privData->octetLen );

	rem = privData->octetLen % 8;
	if (rem>0)  /* No padding needed if rem=0 */
	    for (i=8-rem; i>0; i--, privData->octetLen++ ) 
		*(privData->octs + privData->octetLen) = rand()&0xff;

	des_set_key( privateKey->octs, schedule );
	des_cbc_encrypt( privData->octs, newPrivData->octs, privData->octetLen,
			 schedule, privateKey->octs + 8, DES_ENCRYPT );
	memcpy( privData->octs, newPrivData->octs, privData->octetLen );

	FreeOct( newPrivData );

	return OK ;
}


/* -----------------------------------------------------------------------
 * Function:    DesDecrypt
 * Arguments:   privateKey,    Private key (IN)
 *              privData,      Data that will be decrypted (IN/OUT).
 *		returnvalue    OK if processing was succesfull, neg. otherwise.
 * Purpose:     Decrypts privData according to specified protocol,
 *              as specified in RFC 1446 (Security Protocols for SNMPv2)
 *              section 1.5.2.
 * Remarks:
 * -----------------------------------------------------------------------
 */
	STATUS
DesDecrypt( AsnOcts *privateKey, AsnOcts *privData )
{
	int	rem;
	des_key_schedule schedule;
	AsnOcts	*newPrivData ;

	rem = privData->octetLen % 8;
	if (rem != 0) {
	    	return FAILED ;
	} else {
		newPrivData = NewOct( privData->octetLen + 8 );

	    	des_set_key( privateKey->octs, schedule );
	    	des_cbc_encrypt( privData->octs, newPrivData->octs,
		   privData->octetLen,schedule, privateKey->octs + 8, DES_DECRYPT );
	    	memcpy( privData->octs, newPrivData->octs, privData->octetLen);

		FreeOct( newPrivData );
	}
	return OK ;
}

/* -----------------------------------------------------------------------
 */



