/* -----------------------------------------------------------------------
 *
 * Project:	SNMPV2
 *
 * Filename:    MIB_INT.c
 *
 * Date:        April 1994
 *
 * Purpose:     This file contains routines that can be used to access MIBs.
 *
 * Author( s):  Koert W.G. Flapper
 *
 * -----------------------------------------------------------------------
 */

#include <spmMIB_INT.h>

struct Hashnode  **mibhashtab;
struct MibRecord	**mibtab;
struct TabRecord	**tabtab;
struct MibRecord	*firstmibrec = NULL;
int	mhsize, mhradix, msize, tsize;
int	errno = 0;

/*	Local variables to allow commit and undo for SET requests */
TruthValue	setlistpresent = false;
VarBindList	*SetVarBinds;
VarBind		*restlist;

/*
 * -------------------------------------------------------------------
 *
 * Function:	VarListCopy
 *
 * Arguments:	
 *
 *
 * -------------------------------------------------------------------
 */
	STATUS
VarListCopy( VarBindList *vbl1, VarBindList *vbl2)
{
	char	buf[10000];	/* Hell, I know? */

	(void) DynToStatVbl( vbl1, buf);
	(void) StatToDynVbl( buf, vbl2);
	return(OK);		
}


/*
 * -------------------------------------------------------------------
 *
 * Function:	MibInit
 *
 * Arguments:	
 *
 *
 * -------------------------------------------------------------------
 */
	STATUS
MibInit( submibtab, submsize, subtabtab, subtsize)
struct MibRecord	submibtab[];
struct TabRecord	subtabtab[];
int			submsize, subtsize;
{
	int			i;
	struct MibRecord	*pmr;
	struct Hashnode		*phn;
	int			loc;

	if (mibhashtab == NULL) {
		mhsize = 101;
		mhradix = 13;
		if ((mibhashtab = (struct Hashnode **)malloc(mhsize * sizeof(struct Hashnode *))) == NULL)
			return(ERROR);
		hashclear( mibhashtab, mhsize);
	}
	if (mibtab == NULL) {
		msize = 0;
		if ((mibtab = ( struct MibRecord **)malloc(0)) == NULL)
			return(ERROR);
	}
	if (tabtab == NULL) {
		tsize = 0;
		if ((tabtab = ( struct TabRecord **)malloc(0)) == NULL)
			return(ERROR);
	}

	if ((mibtab = (struct MibRecord **)realloc(mibtab, (msize + submsize)*sizeof(struct MibRecord
*))) == NULL)
		return(ERROR);

	for( i = 0; i < submsize; i++, msize++)
		mibtab[msize] = &submibtab[i];

	if ((tabtab = (struct TabRecord **)realloc(tabtab, (tsize + subtsize)*sizeof(struct TabRecord
*))) == NULL)
		return(ERROR);
	for( i = 0; i < subtsize; i++, tsize++)
		tabtab[tsize] = &subtabtab[i];

	for( i = 0, pmr = &submibtab[submsize-1]; i< submsize; i++, pmr--) {
		loc = hashoid( &pmr->OID, mhsize, mhradix);
		if ((phn = (struct Hashnode *)malloc(sizeof(struct Hashnode))) == NULL)
			return(ERROR);
		phn->pentry = pmr;
		phn->pnext = mibhashtab[loc];
		mibhashtab[loc] = phn;
		pmr->next = firstmibrec;

		if (pmr->vartype == TABLE)
			((struct TabRecord *)pmr->param)->t_mibrec = pmr;
		if (pmr->leaf || (pmr->vartype == TABLE))
			firstmibrec = pmr;
	}
	return(OK);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibVLRequest
 *
 * Arguments:   
 *		reqtype		ReqType	(int) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:      
 *
 * Remarks:	none
 *
 *-------------------------------------------------------------------
 */
	STATUS	
MibVLRequest( VarBindList *mVarBinds, ReqType reqtype, int *error)
{
	VarBind	*vb;

	if (setlistpresent == true)
		if (reqtype == P_COMMIT) {
			pFreeVbl( SetVarBinds);
			setlistpresent = false;
			*error = NOERROR;
			return(OK);
		} else if (reqtype == P_UNDO) {
			mVarBinds = SetVarBinds;
			reqtype = P_SET;
		} else {
			*error = GENERR;
			return(ERROR);
		}
	else if (reqtype == P_SET) {
		SetVarBinds = pNewVbl();
		VarListCopy( mVarBinds, SetVarBinds);
		FOR_EACH_LIST_ELMT(vb, SetVarBinds) {
			if ((vb->varBindChoice->a.value->choiceId == OBJECTSYNTAX_STRING_VALUE) ||
(vb->varBindChoice->a.value->choiceId == OBJECTSYNTAX_OBJECTID_VALUE))
				free( vb->varBindChoice->a.value->a.string_value->octs);
			free( vb->varBindChoice->a.value);
			vb->varBindChoice->choiceId = VARBINDCHOICE_UNSPECIFIED;
			if (MibRequest( vb, P_GET) != OK) {
				*error = NOSUCHNAME;
				return(ERROR);
			}
		}
	SetVarBinds->curr = SetVarBinds->first;
	}
	FOR_EACH_LIST_ELMT( vb, mVarBinds) {
		if (MibRequest( vb, reqtype) == ERROR) {
			/* an error has occurred, stop immediately */
			if ( reqtype == P_SET)
				if (setlistpresent == true) {
					*error = UNDOFAILED;
					SetVarBinds->curr = (void *)restlist;
					pFreeVbl( SetVarBinds);
					setlistpresent = false;
				} else {
					*error = errno;
					/* Only the varbinds that have been executed should be in the
setlist */	
					while (SetVarBinds->curr->prev) {
						SetVarBinds->curr = SetVarBinds->curr->prev;
						vb = (VarBind *)SetVarBinds->curr->data;
						if (MibRequest( vb, P_SET) == ERROR)
							*error = UNDOFAILED;
					}
					pFreeVbl( SetVarBinds);				
					setlistpresent = false;
				}
			return(ERROR);
		}
		if ((reqtype == P_SET) && (setlistpresent == false))
			SetVarBinds->curr = SetVarBinds->curr->next;
	}
	if ( reqtype == P_SET)
		if (setlistpresent == true) {
			SetVarBinds->curr = (void *)restlist;
			pFreeVbl( SetVarBinds);
			setlistpresent = false;
		} else {
			setlistpresent = true;
		}
	*error = NOERROR;
	return(OK);
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibRequest
 *
 * Arguments:   objectID	OID	(in)
 *		object		ObjectSyntax	(out)
 *		reqtype		ReqType	(int) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:      
 *
 * Remarks:	none
 *
 *-------------------------------------------------------------------
 */
	STATUS	
MibRequest( VarBind *varbind, ReqType reqtype)
{
	struct MibRecord	*mibrec;

	if (GetMibRecord( &varbind->name, &mibrec) != OK)
		if (reqtype == P_GET_NEXT) {
			mibrec = firstmibrec;
			while (mibrec) {
				if (CMPOID( &varbind->name, &mibrec->OID) == LESS) {
					reqtype = P_GET_FIRST;
					break;
				}
				mibrec = mibrec->next;
			}
			if (!mibrec) {
				varbind->varBindChoice->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
				return(FAILED);
			}
		} else {
			varbind->varBindChoice->choiceId = VARBINDCHOICE_NOSUCHOBJECT;
			return(FAILED);
		}

	/* a mibrecord has been found */
	if (mibrec->func == 0) {	/* objid is aggregate */
		if ( reqtype != P_GET_NEXT) {
			varbind->varBindChoice->choiceId = VARBINDCHOICE_NOSUCHOBJECT;
			return(FAILED);
		} else {
			if ((mibrec = mibrec->next) == NULL) {
				varbind->varBindChoice->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
				return(FAILED);	/* end of mibview */
			} else	reqtype = P_GET_FIRST;
		}
	} else {	/* objid is table or item */
		if (CMPOID( &varbind->name, &mibrec->OID) == EQUAL)
			if ( mibrec->vartype == TABLE)
				if (reqtype == P_GET_NEXT)
					reqtype = P_GET_FIRST;
	}
	return((mibrec->func)(&varbind->name, varbind->varBindChoice, mibrec, reqtype));
}	

/* 
 * -------------------------------------------------------------------
 *
 * Function:    GetMibRecord
 *
 * Arguments:   oid		*AsnOid	(in)
 *		mr		*MibRecord	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:      
 *
 * Remarks:	none
 *
 *-------------------------------------------------------------------
 */

	STATUS
GetMibRecord( AsnOid *oid, struct MibRecord **mr)
{
	struct Hashnode	*phn;
	int		loc, i;
	unsigned char	tmpinstance;

	tmpinstance = oid->octs[oid->octetLen-1];
	if (tmpinstance == 0)
		oid->octetLen--;
	loc = hashoid( oid, mhsize, mhradix);
	for( phn = mibhashtab[loc]; phn; phn = phn->pnext)
		if ( CMPOID( oid, &((struct MibRecord *)phn->pentry)->OID) == EQUAL) {
			*mr = phn->pentry;
			if (tmpinstance == 0)
				oid->octetLen++;
			return(OK);
		}
	if (tmpinstance == 0)
		oid->octetLen++;
	for( i = 0; i < tsize ; i++)
		if ( !memcmp( oid->octs, tabtab[i]->t_mibrec->OID.octs,
tabtab[i]->t_mibrec->OID.octetLen)) {
			*mr = tabtab[i]->t_mibrec;
			return(OK);
			}
	return(FAILED);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibItemRequest
 *
 * Arguments:   oid		*AsnOid	(in)
 *				*	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:      
 *
 * Remarks:	the oid specifies a valid object identifier possibly 
 *		appended with a '0' instantiation.
 *
 *-------------------------------------------------------------------
 */

	STATUS
MibItemRequest( AsnOid *oid, VarBindChoice *vbc, struct MibRecord *mr, ReqType reqtype)
{
	ObjectSyntax	*objectval;

	if (reqtype == P_GET_NEXT)
		if (oid->octs[oid->octetLen-1] != 0) {
			if ((oid->octs = (unsigned char *)realloc(oid->octs, oid->octetLen+1)) == NULL) {
				errno = RESOURCEUNAVAILABLE;
				return(ERROR);
			}
			oid->octs[oid->octetLen++] = (unsigned char) 0;
			reqtype = P_GET_FIRST;
		} else
			if(mr->next)
				return((mr->next->func)( oid, vbc, mr->next, P_GET_FIRST));
			else {
				vbc->choiceId = VARBINDCHOICE_ENDOFMIBVIEW;
				return(FAILED);
			}
	if (reqtype == P_SET) {
		if (oid->octs[oid->octetLen-1] != 0) {
			errno = NOSUCHINSTANCE;
			return(ERROR);
		}
		if (!mr->writable) {
			errno = NOTWRITABLE;
			return(ERROR);
		}
		switch( mr->vartype) {
		case ASN1_INT:
			if (vbc->a.value->choiceId != OBJECTSYNTAX_INTEGER_VALUE) {
				errno = WRONGTYPE;
				return(ERROR);
			}
			if (mr->param == 0)
				return(ERROR);
			*((AsnInt *) mr->param) = vbc->a.value->a.integer_value;
			break;
		case ASN1_OCTSTR:
			if (vbc->a.value->choiceId != OBJECTSYNTAX_STRING_VALUE) {
				errno = WRONGTYPE;
				return(ERROR);
			}
			((AsnOcts *)mr->param)->octetLen = vbc->a.value->a.string_value->octetLen;
			memcpy( ((AsnOcts *)mr->param)->octs, vbc->a.value->a.string_value->octs,
((AsnOcts *)mr->param)->octetLen);
			break;
		case ASN1_OID:
			if (vbc->a.value->choiceId != OBJECTSYNTAX_OBJECTID_VALUE) {
				errno = WRONGTYPE;
				return(ERROR);
			}
			((AsnOid *)mr->param)->octetLen = vbc->a.value->a.objectID_value->octetLen;
			memcpy( ((AsnOid *)mr->param)->octs, vbc->a.value->a.objectID_value->octs,
((AsnOid *)mr->param)->octetLen);
			break;
		}
		return(OK);
	}
	if (reqtype == P_GET_FIRST) {
		if ((oid->octs = (unsigned char *)realloc(oid->octs, mr->OID.octetLen+1)) == NULL) {
			errno = RESOURCEUNAVAILABLE;
			return(ERROR);
		}
		oid->octetLen = mr->OID.octetLen;
		memcpy( oid->octs, mr->OID.octs, oid->octetLen);
		oid->octs[oid->octetLen++] = (unsigned char) 0;
	}

	/* operation is Get or GetFirst */
	if (oid->octs[oid->octetLen-1] != 0) {
		vbc->choiceId = VARBINDCHOICE_NOSUCHINSTANCE;
		return(FAILED);
	}
	if ((objectval = (ObjectSyntax *)malloc(sizeof(ObjectSyntax))) == NULL) {
		errno = RESOURCEUNAVAILABLE;
		return(ERROR);
	}
	vbc->a.value = objectval;
	vbc->choiceId = VARBINDCHOICE_VALUE;

	switch( mr->vartype) {
		case ASN1_INT:
		case ASN1_TIMETICKS:
		case ASN1_GAUGE:
		case ASN1_COUNTER:
			vbc->a.value->choiceId = OBJECTSYNTAX_INTEGER_VALUE;
			vbc->a.value->a.integer_value = *((int *)mr->param);
			break;
		case ASN1_OCTSTR:
			vbc->a.value->choiceId = OBJECTSYNTAX_STRING_VALUE;
			vbc->a.value->a.string_value = ((AsnOcts *)mr->param);
			break;
		case ASN1_OID:
			vbc->a.value->choiceId = OBJECTSYNTAX_OBJECTID_VALUE;
			vbc->a.value->a.objectID_value = ((AsnOid *)mr->param);
			break;
	}
	return(OK);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    MibTableRequest
 *
 * Arguments:   oid		*AsnOid	(in)
 *				*	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:      
 *
 * Remarks:	none
 *
 *-------------------------------------------------------------------
 */

	STATUS
MibTableRequest( AsnOid *oid, VarBindChoice *vbc, struct MibRecord *mr, ReqType reqtype)
{
	ObjectSyntax	*objectval;

	if (reqtype != P_SET) {
		if ((objectval = (ObjectSyntax *)malloc(sizeof(ObjectSyntax))) == NULL) {
			errno = RESOURCEUNAVAILABLE;
			return(ERROR);
		}
		vbc->a.value = objectval;
		vbc->choiceId = VARBINDCHOICE_VALUE;
	}
	switch(reqtype) {

	case P_GET:
		return(((struct TabRecord *)mr->param)->t_get)(oid, vbc);
	case P_GET_FIRST:
		return(((struct TabRecord *)mr->param)->t_getf)(oid, vbc, mr);
	case P_GET_NEXT:
		return(((struct TabRecord *)mr->param)->t_getn)(oid, vbc, mr);
	case P_SET:
		return(((struct TabRecord *)mr->param)->t_set)(oid, vbc);
	default:
		return(ERROR);
	}
}





