/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:    spmMain.c
 *
 * Date:        May 1994
 *
 * Purpose:     The main program of the SnmpV2 Protocol Machine (SPM)
 *
 * Author(s):   Vincent Berkhout, Frederic P. H. van Hengstum
 *
 * -----------------------------------------------------------------------
 */

#include <spmMain.h>


/* 
 *  -----------------------------------------------------------------------
 *
 * Function:	OpenSNMPv2ProtocolMachine
 *
 * Arguments:   none.
 *
 * Return:	
 *
 * Purpose:	initialize all the  SNMPv2 Protocol Machine modules.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
OpenSNMPv2ProtocolMachine( void )
{
	if( TraceInit( ) != OK ) {
		Error( "unable to open Trace module") ;
		return FAILED ;
	}

	TRACE( SNMP_DEBUG_MAIN, "starting Initialization" );

	if ( SpmInitPartyMIB() != OK ) {
		TRACE( SNMP_DEBUG_MAIN, "unable to open PartyMib module" );
                return ERROR ;
        }
	if ( SpmInitSnmpV2Mib() == ERROR ) {
		TRACE( SNMP_DEBUG_MAIN, "unable to open SnmpV2Mib module" );
                return FAILED ;
        }


	if (SpmInitAim () != OK ) {
		TRACE( SNMP_DEBUG_MAIN, "unable to open AIM" );
                return ERROR ;
        }

	if (SpmInitTim () != OK ) {
		TRACE( SNMP_DEBUG_MAIN, "unable to open TIM" );
                return ERROR ;
        }

	if (SpmInitReceive() != OK ) {
		TRACE( SNMP_DEBUG_MAIN, "unable to open RECEIVE modules" );
                return FAILED ;
        }

	if (SpmInitSend() != OK ) {
		TRACE( SNMP_DEBUG_MAIN, "unable to open SEND modules" );
                return FAILED ;
        }

	if (SpmInitBer () != OK ) {
		TRACE( SNMP_DEBUG_MAIN, "unable to open CM modules" );
                return FAILED ;
        }

	if (SpmInitAdminMib () != OK ) {
		TRACE( SNMP_DEBUG_MAIN, "unable to open AdminMib modules" );
                return FAILED ;
        }

	TRACE( SNMP_DEBUG_MAIN, "Initialization done" );

	return OK ;

}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    CloseSNMPv2ProtocolMachine
 *
 * Arguments:   none.
 *
 * Return:
 *
 * Purpose:     Close the Protocol Machine gracefull down.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CloseSNMPv2ProtocolMachine()
{
	AimClose ( "spm" );

	SaveSnmpStats();

	/* SavePartyMIB() ;

	CloseTransportConnections(); */ 	/* might be obsolete! */

        return OK ;
}


/*  
 * -----------------------------------------------------------------------
 *
 * Function:    ProcessEventsInProtocolMachine
 *
 * Arguments:   none.
 *
 * Return:
 *
 * Purpose:     main routine in the SNMPv2 Protocol Machine, who dispaches
 *		the different events over the appropriate routines.
 *
 * Remarks:	Only terminate the loop on CRITICAL ERRORS !!!!!!!!!
 *
 * -----------------------------------------------------------------------
 */
	STATUS
ProcessEventsInSNMPv2ProtocolMachine( )
{
	BOOLEAN	continu = TRUE;

	do {
		TraceUser();
		/*
		 * Step 1: process all input coming from the various applications
		 */
		if ( SpmSend( ) == ERROR )
			return ERROR ;

		/*
		 * Step 2: process all packets coming from the transportlayer
		 */
		if ( SpmReceive( ) == ERROR )
			return ERROR ;	

	} while( continu ) ;
	
	return OK ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    main program
 *
 * Arguments:   argc, argument count (IN).
 *		argv, argument vector (IN).
 *		arge, argument environment (IN).
 *
 * Return:
 *
 * Purpose:     Routine to start, run and stop the SNMPv2 Protocol Machine.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
main( argc, argv, arge ) 
	int argc;
	char **argv;
	char **arge;
{
	if ( OpenSNMPv2ProtocolMachine( ) == ERROR ) {
		(void) CloseSNMPv2ProtocolMachine( );
		exit( FAILED ) ;
	}

	if ( ProcessEventsInSNMPv2ProtocolMachine( ) == ERROR ) {
		(void) CloseSNMPv2ProtocolMachine( );
		exit( FAILED ) ;
	}

	(void) CloseSNMPv2ProtocolMachine( );

	exit (OK) ; 
}


/* ------------------------ end of file spmMain.c ------------------------*/

