/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:    spmTrace.c
 *
 * Date:        May 1994
 *
 * Purpose:     This file implements the debug/trace handling.
 *
 * Author(s):   Anton Api Holleman, 
 *		Vincent Berkhout, Frederic P. H. van Hengstum
 *
 * -----------------------------------------------------------------------
 */

#include <snmpTypes.h>

#define MaxBuf 64

#define MaxTraceTable 128
#define MaxTraceFile 128
static struct {
	char active;
	char tag[MaxBuf];
	char comment[MaxBuf];
} TraceTable[MaxTraceTable];


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    UserMonitor
 *
 * Arguments:   
 *
 * Purpose:     
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
        STATUS
UserMonitor( )
{
	fd_set fds;
	struct timeval tv;

	tv.tv_sec = 0;
	tv.tv_usec = 10;

	FD_ZERO( &fds );
	FD_SET( 0, &fds );

	if( select( getdtablesize(), &fds, 0, 0, &tv ) > 0 )
		return( OK );
	return( FAILED );
}

/* 
 * -----------------------------------------------------------------------
 *
 * Function:    Trace
 *
 * Arguments:   
 *
 * Purpose:     
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
Trace( va_alist ) va_dcl
{
    	va_list args ;
    	char    *format ;
	int number;

	va_start( args );
	number = va_arg( args, int );
	if( TraceTable[number].active != ' ' )
		return FAILED ;
	(void) fprintf( stderr, "%s:", TraceTable[number].tag );
	format = va_arg( args, char * ) ;
	(void) vfprintf( stderr, format, args );
	(void) fprintf( stderr, ".\n" ) ;
	va_end(args) ;
	return OK ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    TraceInit
 *
 * Arguments:   
 *
 * Purpose:     
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
TraceInit( )
{
	short i;

	for( i = 0; i < MaxTraceTable; i++ ) {
		TraceTable[i].active = 0x00;
		memset( TraceTable[i].tag, 0x00, MaxBuf );
		memset( TraceTable[i].comment, 0x00, MaxBuf );
	}
	traceLoad( );
	return OK;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    TraceLoad
 *
 * Arguments:   
 *
 * Purpose:     
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
traceLoad( )
{
	char TraceFile[MaxTraceFile] = "./trace.conf" ;
	FILE *fp;
	int i, size;
	char *p, buf[MaxBuf], tag[MaxBuf], comment[MaxBuf];

	
	if( getenv( "UT_SNMP_TRACE" ) )
		strcpy( (char *) TraceFile, 
			(char *) getenv( "UT_SNMP_TRACE" ) );

	if( (fp = fopen( TraceFile, "r" )) == NULL )
		return FAILED;

	i = 0;
	(void) memset( buf, 0x00, MaxBuf );
	(void) memset( tag, 0x00, MaxBuf );
	(void) memset( comment, 0x00, MaxBuf );

	while( fgets( buf, MaxBuf, fp ) ) {
		if( ( size = strlen( buf ) ) > 1 ) {
			p = buf;
			if( buf[0] == '!' ) p = &buf[1];
			sscanf( p, "%d %s %s", &i, tag, comment );
			if( (i >= 0) && (i < MaxTraceTable) ) {
				if( buf[0] == '!' )
					 TraceTable[i].active = '!';
				else
					 TraceTable[i].active = ' ';
				strcpy( TraceTable[i].tag, tag );
				strcpy( TraceTable[i].comment, comment );
			}
			(void) memset( buf, 0x00, MaxBuf );
			(void) memset( tag, 0x00, MaxBuf );
			(void) memset( comment, 0x00, MaxBuf );
		}
	}
	fclose( fp );
	return OK;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    Trace[In]Active
 *
 * Arguments:   
 *
 * Purpose:     
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
traceActive( int i ) 
{
	TraceTable[i].active = ' ';
	return OK ;
}


	STATUS
traceInActive( int i )
{
	TraceTable[i].active = '!';
	return OK ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    TraceTrace
 *
 * Arguments:   
 *
 * Purpose:     
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
OverviewExt( )
{
	short i;

	fprintf( stderr, "\n\tUT-SNMP TRACETABLE.\n\n" );
	fprintf( stderr, 
	" =======================================================================\n" );
	fprintf( stderr, " Active\tNumber\tTag\t\t\tDescription\n" );
	fprintf( stderr, " ------\t------\t---\t\t\t-----------\n" );
	for( i = 0; i < MaxTraceTable; i++ )
		if( TraceTable[i].active != (char) 0x00 )
		{
			if ( strcmp( "not_yet_impl", TraceTable[i].tag) ) 
				fprintf( stderr, " %c\t%d\t%s\t\t%s\n", TraceTable[i].active,
				i, TraceTable[i].tag , TraceTable[i].comment);
		}
	fprintf( stderr, 
	" =======================================================================\n" );
	return( OK );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    trace
 *
 * Arguments:   
 *
 * Purpose:     
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	void
traceInt( )
{ 
	char buf[MaxBuf];
	int i;
	int stop = FAILED;

	system("clear");
	while( stop ) {
		OverviewExt( ); 
		fprintf( stderr, "\n\tTrace menu.\n\t-----------\n\n" );
		fprintf( stderr, "\tE(nable)<number><enter>\n" );
		fprintf( stderr, "\tD(isable)<number><enter>\n" );
		fprintf( stderr, "\tM(onitor)<enter>\n" );
		fprintf( stderr, "\tC(rash spm) or Q(uit)<enter>\n\n\tOption: " );
		fflush( stderr );
		fscanf( stdin, "%s", buf );
		switch( *buf ) {
		case 'e': case 'E': 
			i=atoi(&buf[1]); 
			traceActive( i );
			break;
		case 'd': case 'D': 
			i=atoi(&buf[1]); 
			traceInActive( i );
			break;
		case 'm': case 'M': 
			stop = OK;
			system("clear");
			break;
		case 'q': case 'Q': 
		case 'c': case 'C': 
			exit(0); 
			break;
		default: 
			fprintf( stderr, "\tError: Invalid command !!!\n" );
			break;
		}
	}
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    main
 *
 * Arguments:   
 *
 * Purpose:     
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
TraceUser( )
{
	if( UserMonitor( ) == OK )
		traceInt();
	return OK ;
}
/*
	STATUS
main( int argc, char** argv, char** arge )
{
	char *apiname = "API";
	int filenum = 5;

	(void) traceInit( );
	if( traceLoad( ) )
		exit( -1 );

	traceInt();
	traceInActive( 1 );
	traceActive( 1 );

	Error( "voorbeeld foutje %s in %s", "bedankt", apiname, NULL, 
		NULL );

	while( 1 )
		if( UserMonitor( ) == OK )
			traceInt();

	return OK ;
}
*/


/* -----------------------------------------------------------------------
 */
